package Ajout_Modif_Chambres;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

public class AM_Chambres_Frame extends JFrame {
	
	private static final long serialVersionUID = 1L;
	private Locale locale;
	@SuppressWarnings("unused")
	private String db_link;
	private JMenuBar menu_bar = new JMenuBar();
	private JMenu file_menu = new JMenu();
	private JMenuItem close_menu_item = new JMenuItem();
	
	
	private JLabel j = new JLabel();
	private JLabel k = new JLabel();
	private JLabel l = new JLabel();
	private JLabel m = new JLabel();
	
	private JPanel cb_panel = new JPanel(new FlowLayout());
	private JPanel info_panel = new JPanel(new FlowLayout());
	private JPanel exec_panel = new JPanel(new FlowLayout());
	private AM_Chambres_Frame self = this;
	private JComboBox<String> cb_name = new JComboBox<String>();
	private JButton delete = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/close.png")));
	private AM_Chambres_Query am_query;
	private JTextField tf_name = new JTextField();
	private JTextField tf_surface_ch = new JTextField();
	private JTextField tf_surface_cp = new JTextField();
	private JTextArea desc = new JTextArea(5,20);
	private JButton am_button = new JButton();
	private enum Mode { AJOUT, MODIFICATION, DISABLE };
	private Mode button_mode = Mode.DISABLE;
	
	public AM_Chambres_Frame(Locale locale, String db_link)
	{
		super();
		this.locale = locale;
		this.db_link = db_link;
		
		am_query = new AM_Chambres_Query(db_link);
		
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.setLayout(new BorderLayout());
		this.translate(locale);
		this.add_menu();
		this.add_buttons();
		this.add_handlers();
		this.add_name();
		this.draw_frame();
	}
	
	/**
	 * Affiche les infos d'une chambre
	 * @param name nom de la chambre
	 */
	private void show_info(String name)
	{
		ArrayList<String> info = am_query.get_info(name);
		
		// Pour la traduction du bouton "Modifier la chambre"
		ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		
		
		tf_name.setText(info.get(0));
		tf_surface_ch.setText(info.get(1));
		tf_surface_cp.setText(info.get(2));
		desc.setText(info.get(3));
		
		tf_name.setEditable(true);
		tf_surface_ch.setEditable(true);
		tf_surface_cp.setEditable(true);
		desc.setEditable(true);
		am_button.setText(rb.getString("modifChambre"));
		button_mode = Mode.MODIFICATION;
		am_button.setEnabled(true);
		delete.setEnabled(true);
	}
	
	/**
	 * Remplit cb_name avec tous les noms de chambres
	 */
	private void add_name()
	{
		
		// seulement pour la traduction de l'item "Nouveau"
		ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);

		cb_name.removeAllItems();
		cb_name.addItem("");
		for( String s : am_query.get_chambres_name() )
			cb_name.addItem(s);
		cb_name.addItem(rb.getString("nouveau"));
	}
	
	/**
	 * Supprimer une chambre
	 * @param name nom de la chambre
	 */
	@SuppressWarnings("static-access")
	private void delete_ch(String name, ResourceBundle rb)
	{
		// bote de dialogue : confirmation de suppression d'une chambre donne
		if( JOptionPane.showConfirmDialog(self, rb.getString("mssgConfirmSupprCh"), 
				rb.getString("titreBoxConfirmSupprCh"), JOptionPane.WARNING_MESSAGE) 
				== JOptionPane.OK_OPTION) {
			
			// suppression de la chambre
			if ( ! am_query.delete_ch(name) )
				// si erreur de suppression : affichage d'une bote de dialogue d'avertissement
				new JOptionPane().showMessageDialog(self, rb.getString("mssgErrSupprCh"),
						rb.getString("titreBoxErrSupprCh"), JOptionPane.ERROR_MESSAGE);
			
			// mise  jour des noms de chambre dans la liste
			this.add_name();
		}
	}
	
	private void add_menu()
	{
		file_menu.add(close_menu_item);
		menu_bar.add(file_menu);
		this.setJMenuBar(menu_bar);
	}
	
	private void add_handlers()
	{
		
		// pour la traduction de l'item "Nouveau"
		final ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		
		// bouton fermer
		close_menu_item.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						am_query.close();
						self.dispose();
					}});
		
		// liste droulante permettant le choix de la chambre
		cb_name.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						@SuppressWarnings("unchecked")
						String selected = ((String)((JComboBox<String>)e.getSource()).getSelectedItem());
						if( selected != null )
						{
							// si une chambre existante a t selectionne
							if( ! selected.isEmpty() && selected.compareTo(rb.getString("nouveau")) !=0 )
								self.show_info(selected);
							else 
							{
								// si "Nouveau" est choisi dans la liste des chambres
								if( selected.compareTo(rb.getString("nouveau")) == 0)
								{
									delete.setEnabled(false);
									// prparation des zones de saisies
									tf_name.setText("");
									tf_surface_ch.setText("");
									tf_surface_cp.setText("");
									desc.setText("");
									tf_name.setEditable(true);
									tf_surface_ch.setEditable(true);
									tf_surface_cp.setEditable(true);
									desc.setEditable(true);
									button_mode = Mode.AJOUT;
									am_button.setText(rb.getString("ajoutChambre"));
									am_button.setEnabled(true);
								}
								else
									// si rien n'a t selectionn dans la liste de chambres
									if( selected.isEmpty())
									{
										tf_name.setText("");
										tf_surface_ch.setText("");
										tf_surface_cp.setText("");
										desc.setText("");
										delete.setEnabled(false);
										tf_name.setEditable(false);
										tf_surface_ch.setEditable(false);
										tf_surface_cp.setEditable(false);
										desc.setEditable(false);
										button_mode = Mode.DISABLE;
										am_button.setText((rb.getString("choisirChambre")));
										am_button.setEnabled(false);	
									}
							}
						}
					}});
		
		// bouton de suppression d'une chambre
		delete.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						self.delete_ch((String)cb_name.getSelectedItem(), rb);
					}});
		
		// modification d'une chambre
		am_button.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						self.modif_ch(rb);
					}});
		}
	
	/**
	 * Modifie les infos de la chambre selectionne
	 */
	@SuppressWarnings("static-access")
	private void modif_ch(ResourceBundle rb)
	{
		// liste pour contenir les informations de la chambre
		ArrayList<String> info = new ArrayList<String>();
		
		// si toutes les informations sont renseignes
		if( ! tf_name.getText().trim().isEmpty() && ! tf_surface_ch.getText().trim().isEmpty()
				&& ! tf_surface_cp.getText().trim().isEmpty() && ! desc.getText().trim().isEmpty() )
		{
			// ajout des informations dans la liste info
			info.add(tf_name.getText().trim());
			try
			{
				info.add(Float.valueOf(tf_surface_ch.getText()).toString());
				info.add(Float.valueOf(tf_surface_cp.getText()).toString());
				info.add(desc.getText().trim());
				info.add((String)cb_name.getSelectedItem());
				
				// si la chambre existait dj : on la modifie
				if( button_mode == Mode.MODIFICATION )
					am_query.modif_ch(info);
				// sinon, on l'ajoute
				else
					if (button_mode == Mode.AJOUT)
						am_query.new_ch(info);
				
				// on met  jour les noms des chambres dans la liste droulante
				this.add_name();
			}
			catch(NumberFormatException f)
			{
				// boite de dialogue d'erreur si le champ surface est au mauvais format
				new JOptionPane().showMessageDialog(self, rb.getString("mssgErrSurf"),
						rb.getString("titreBoxErrSurf"), JOptionPane.ERROR_MESSAGE);

			}	
		}
		// s'il manque certaines informations : affichage d'une bote de dialogue
		else
			new JOptionPane().showMessageDialog(self, rb.getString("mssgChampsVides"),
					rb.getString("titreBoxChampsVides"), JOptionPane.ERROR_MESSAGE);
	}
	
	private void add_buttons()
	{
		delete.setPreferredSize(new Dimension(20,20));
		delete.setEnabled(false);
		
		cb_panel.add(cb_name);
		cb_panel.add(delete);
		cb_panel.setBackground(new Color(192, 197, 199));
		this.getContentPane().add(cb_panel, BorderLayout.NORTH);
		
		tf_name.setPreferredSize(new Dimension(193,20));
		tf_name.setEditable(false);
		tf_surface_ch.setPreferredSize(new Dimension(74,20));
		tf_surface_ch.setEditable(false);
		tf_surface_cp.setPreferredSize(new Dimension(50,20));
		tf_surface_cp.setEditable(false);
		
		JScrollPane scrollPane = new JScrollPane(desc);
		desc.setLineWrap(true);
		desc.setWrapStyleWord(true);
		desc.setEditable(false);
		
		
		((FlowLayout)info_panel.getLayout()).setAlignment(FlowLayout.CENTER);
		info_panel.setPreferredSize(new Dimension(250,160));
		j.setHorizontalAlignment(JLabel.RIGHT);
		info_panel.add(j);
		info_panel.add(tf_name);
		k.setHorizontalAlignment(JLabel.RIGHT);
		info_panel.add(k);
		info_panel.add(tf_surface_ch);
		l.setHorizontalAlignment(JLabel.RIGHT);
		info_panel.add(l);
		info_panel.add(tf_surface_cp);
		m.setHorizontalAlignment(JLabel.RIGHT);
		info_panel.add(m);
		info_panel.add(scrollPane);
		info_panel.setBackground(new Color(192, 197, 199));
		this.getContentPane().add(info_panel, BorderLayout.CENTER);
		
		am_button.setEnabled(false);
		exec_panel.add(am_button);
		exec_panel.setBackground(new Color(192, 197, 199));
		this.getContentPane().add(exec_panel, BorderLayout.SOUTH);
		
	}
	
	private void draw_frame()
	{
		this.setResizable(false);
		this.setLocation(200,200);
		this.pack();
		this.setVisible(true);
	}
	
	/**
	 * Traduction dans la bonne langue des textes de la fentre
	 * @param locale
	 */
	public void translate(Locale locale){
		this.locale = locale;
		ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		
		// nom de la fentre
		this.setTitle(rb.getString("titreFenAjoutModifChambres"));
		
		file_menu.setText(rb.getString("fichier"));
		close_menu_item.setText(rb.getString("fermer"));
		
		// label nom
		j.setText(rb.getString("champNom"));
		
		// label surface chambre
		k.setText(rb.getString("champSChbre"));
		
		// label surface champ
		l.setText(rb.getString("champSChp"));
		
		// label description
		m.setText(rb.getString("champDescription"));
		
		// bouton choix de chambre ( : texte initial du bouton)
		am_button.setText(rb.getString("choisirChambre"));
		
		//ragencement de la position des items de la fentre
		this.pack();
	}
	
	
}
