package Ajout_Modif_Sites;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;


public class AM_Site_Frame extends JFrame {

	private static final long serialVersionUID = 1L;
	private Locale locale;
	
	private String db_link;
	
	private AM_Site_Frame self = null;
	
	private JMenuBar menu_bar = new JMenuBar();
	private JMenu file_menu = new JMenu();
	private JMenuItem close_menu_item = new JMenuItem();
	
	private JLabel site_lbl = new JLabel();
	
	private JTable liste = null;
	
	private JButton modif_entry = new JButton();
	private JButton delete = new JButton(new ImageIcon(getClass().getClassLoader().getResource("images/close.png")));
	
	private JTextField name_filter = new JTextField();
	
	private AM_Site_Query am_query;
	
	
	public AM_Site_Frame(Locale locale, String db_link)
	{
		super();
		self = this;
		this.locale = locale;
		this.db_link = db_link;
		
		am_query = new AM_Site_Query(db_link);
		
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.getContentPane().setLayout(new BoxLayout(this.getContentPane(), BoxLayout.LINE_AXIS));
		
		this.translate(locale);
		// traduction des colonnes de la table
		ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		String[] column_name = { rb.getString("colNomSite"), rb.getString("colNomStation"), 
				rb.getString("colMilieu") };
		
		
		this.add_menu();
		this.add_buttons(column_name);
		this.add_handlers(column_name);
		this.draw_frame();
	}
	
	private void add_menu()
	{
		file_menu.add(close_menu_item);
		menu_bar.add(file_menu);
		this.setJMenuBar(menu_bar);
	}

	private void add_handlers(final String[] column_name)
	{
		close_menu_item.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						self.dispose();
						self.am_query.close();
					}});
		
		// gestion de l'auto-slection du nom lorsque l'on commence  taper dans le champ du nom
		name_filter.addKeyListener(
				new KeyListener() {
					public void keyPressed(KeyEvent arg0) {}
					
					public void keyReleased(KeyEvent arg0) {
						// rcupration du texte saisi dans le champ
						String text = ((JTextField)arg0.getSource()).getText();
						if( ! text.isEmpty() )
							// parcours de toute la liste contenant les sites
							for( int i = 0; i < liste.getRowCount(); i++ )
								// si on trouve dans la liste un site qui commence par le texte saisi
								if ( ((String)liste.getValueAt(i, 0)).startsWith(text.toLowerCase()) 
										|| ((String)liste.getValueAt(i, 0)).startsWith(text.toUpperCase()) 
										|| ((String)liste.getValueAt(i, 0)).startsWith(text) ) {
									// alors la ligne correspondante est slectionne
									self.liste.scrollRectToVisible(self.liste.getCellRect(i, 0, true));
									self.liste.getSelectionModel().setSelectionInterval(i, i);
									break;
								}
					}
					public void keyTyped(KeyEvent arg0) {}	
				});
		
		// modification d'un site slectionn
		modif_entry.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent e)
					{
						if( self.liste.getSelectedRow() != -1 )
						{
							final String s_n = (String)self.liste.getValueAt(self.liste.getSelectedRow(), 0);
							final String st_n = (String)self.liste.getValueAt(self.liste.getSelectedRow(), 1);
							final String m = (String)self.liste.getValueAt(self.liste.getSelectedRow(), 2);
							new Modify_Frame(self, s_n, st_n, m, locale, db_link);
							((DefaultTableModel)self.liste.getModel()).setDataVector(am_query.get_data(), column_name);
							liste.getColumnModel().getColumn(0).setPreferredWidth(liste.getColumnModel().getColumn(0).getWidth() * 2 + 60);
							liste.getColumnModel().getColumn(1).setPreferredWidth(liste.getColumnModel().getColumn(1).getWidth() * 2 + 20);
							liste.getColumnModel().getColumn(2).setPreferredWidth(liste.getColumnModel().getColumn(2).getWidth() * 2 - 80);
						}
					}});
		
		// suppression d'un site slectionn
		delete.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent e) 
					{
						if( self.liste.getSelectedRow() != -1 )
						{
							String site_name = (String)self.liste.getValueAt(self.liste.getSelectedRow(), 0);
							String station_name = (String)self.liste.getValueAt(self.liste.getSelectedRow(), 1);
							self.am_query.delete_station(site_name.replaceAll("'", "\\\\'"), station_name.replaceAll("'", "\\\\'"));
							((DefaultTableModel)self.liste.getModel()).setDataVector(am_query.get_data(), column_name);
						}
					}});
	}

	private void add_buttons(final String[] column_name)
	{

		JPanel all = new JPanel();
		all.setLayout(new BoxLayout(all, BoxLayout.PAGE_AXIS));
		site_lbl.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		all.add(Box.createVerticalStrut(5));
		all.add(site_lbl);
		name_filter.setMaximumSize(new Dimension(200,20));
		
		JPanel buttons_pane = new JPanel();
		buttons_pane.setLayout(new BoxLayout(buttons_pane, BoxLayout.LINE_AXIS));
		buttons_pane.setAlignmentX(Component.LEFT_ALIGNMENT);
		delete.setPreferredSize(new Dimension(20,20));
		modif_entry.setPreferredSize(new Dimension(80,20));
		name_filter.setPreferredSize(new Dimension(260,20));
		buttons_pane.setBackground(new Color(192, 197, 199));
		buttons_pane.add(name_filter);
		buttons_pane.add(Box.createHorizontalGlue());
		buttons_pane.add(modif_entry);
		buttons_pane.add(Box.createHorizontalStrut(5));
		buttons_pane.add(delete);
		
		all.add(Box.createVerticalStrut(5));
		all.add(buttons_pane);
		
		liste = new JTable(new DefaultTableModel())
		{
			private static final long serialVersionUID = 1L;
			public boolean isCellEditable(int rowIndex, int colIndex )
			{ return false; }
		
		
			};
		
		// remplissage de la table
		((DefaultTableModel)self.liste.getModel()).setDataVector(am_query.get_data(), column_name);
		liste.getTableHeader().setReorderingAllowed(false);
		liste.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		liste.getTableHeader().setResizingAllowed(false);
		
		//cration possibilit de tri sur entte colonne
		liste.setAutoCreateRowSorter(true);
		
		// ajustement de la largeur des colonnes de la table
		liste.getColumnModel().getColumn(0).setPreferredWidth(liste.getColumnModel().getColumn(0).getWidth() * 2 + 60);
		liste.getColumnModel().getColumn(1).setPreferredWidth(liste.getColumnModel().getColumn(1).getWidth() * 2 + 20);
		liste.getColumnModel().getColumn(2).setPreferredWidth(liste.getColumnModel().getColumn(2).getWidth() * 2 - 80);
		
		JScrollPane liste_scroll = new JScrollPane(liste);
		liste_scroll.setAlignmentX(Component.LEFT_ALIGNMENT);
		liste_scroll.setPreferredSize(new Dimension(600,400));
		all.add(Box.createVerticalStrut(5));
		all.add(liste_scroll);
		all.add(Box.createVerticalStrut(5));
		
		all.setBackground(new Color(192, 197, 199));	
		
		this.getContentPane().add(Box.createHorizontalStrut(5));
		this.getContentPane().add(all);
		this.getContentPane().add(Box.createHorizontalStrut(5));
		this.getContentPane().setBackground(new Color(192, 197, 199));
	
		// ouverture de la fentre de modification d'un site choisi avec un double clique	
		liste.addMouseListener(new MouseAdapter()
		{
			public void mousePressed(MouseEvent e)
			{
				if (e.getClickCount() == 2) {
					final String s_n = (String)self.liste.getValueAt(self.liste.getSelectedRow(), 0);
					final String st_n = (String)self.liste.getValueAt(self.liste.getSelectedRow(), 1);
					final String m = (String)self.liste.getValueAt(self.liste.getSelectedRow(), 2);
					new Modify_Frame(self, s_n, st_n, m, locale, db_link);
					((DefaultTableModel)self.liste.getModel()).setDataVector(am_query.get_data(), column_name);
					liste.getColumnModel().getColumn(0).setPreferredWidth(liste.getColumnModel().getColumn(0).getWidth() * 2 + 60);
					liste.getColumnModel().getColumn(1).setPreferredWidth(liste.getColumnModel().getColumn(1).getWidth() * 2 + 20);
					liste.getColumnModel().getColumn(2).setPreferredWidth(liste.getColumnModel().getColumn(2).getWidth() * 2 - 80);

				}
			}
		});	
	}

	private void draw_frame()
	{
		this.setResizable(false);
		this.setLocation(200,200);
		this.pack();
		this.setVisible(true);
	}
	
	/**
	 * Traduction des textes dans la bonne langue
	 * @param locale
	 */
	public void translate(Locale locale){
		this.locale = locale;
		ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		
		// nom de la fentre
		this.setTitle(rb.getString("titreFenConsultModifSites"));
		
		file_menu.setText(rb.getString("fichier"));
		close_menu_item.setText(rb.getString("fermer"));
		
		site_lbl.setText(rb.getString("triParSite"));
		
		modif_entry.setText(rb.getString("modifier"));
		
		//ragencement de la position des items de la fentre
		this.pack();
	}
	
	

}
