package C_M_Perso;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.print.PrinterException;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JTable.PrintMode;
import javax.swing.ListSelectionModel;
import javax.swing.RowSorter;
import javax.swing.SortOrder;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

public class Consultation_Perso_Frame extends JFrame {
	private static final long serialVersionUID = 1L;
	private Locale locale;
	private String db_link;
	
	private JMenuBar menu_bar = new JMenuBar();
	private JMenu fichier = new JMenu();
	private JMenuItem imprimer = new JMenuItem();
	private JMenuItem fermer = new JMenuItem();
	private Consultation_Perso_Frame self = null;
	private JTable table = new JTable();
	private Consultation_Perso_Query c_query;
	private JTextField code_filter = new JTextField();
	private JTextField nom_filter = new JTextField();

	private JLabel recherche_lbl = new JLabel();
	
	public Consultation_Perso_Frame(Locale locale, String db_link)
	{
		super();
		self = this;
		this.locale = locale;
		this.db_link = db_link;
		
		c_query = new Consultation_Perso_Query(db_link);
		
		this.setLayout(new BorderLayout());
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
		add_menu();
		add_table();
		
		this.translate(locale);
		
		add_handlers();
		show_frame();	
	}

	private void add_menu()
	{
		fichier.add(imprimer);
		fichier.add(fermer);
		menu_bar.add(fichier);
		this.setJMenuBar(menu_bar);
	}

	private void add_handlers()
	{
		// traduction du titre sur la page d'impression
		final ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		
		fermer.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						self.dispose();
					}});

		imprimer.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						MessageFormat header = new MessageFormat(rb.getString("listeTaxonsPerso"));
						MessageFormat footer = new MessageFormat(new SimpleDateFormat("dd/MM/yyyy").format(new java.util.Date()) + "                                " +
								rb.getString("phytobsVer") + "                        " +
								"                    {0}");
						try {
							if (table.print(PrintMode.FIT_WIDTH, header, footer) )
								self.dispose();
						} catch (PrinterException e1) {e1.printStackTrace();}
					}});
		
		//recherche par code 6 lettres	
		code_filter.addKeyListener(
				new KeyListener() {
					public void keyPressed(KeyEvent arg0) {}
					public void keyReleased(KeyEvent arg0) {
						String text = ((JTextField)arg0.getSource()).getText();
						if( ! text.isEmpty() )
						{
							nom_filter.setText("");
							// tri par la colonne code taxon
							List<RowSorter.SortKey> keys = new ArrayList<RowSorter.SortKey>();
							keys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
							table.getRowSorter().setSortKeys(keys);
							
							for( int i = 0; i < table.getRowCount(); i++ )
								if( ((String)table.getValueAt(i, 0)).toUpperCase().startsWith(text.toUpperCase()))
								{
									self.table.scrollRectToVisible(self.table.getCellRect(i+5, 0, true));
									self.table.getSelectionModel().setSelectionInterval(i, i);
									break;
								}
						}
					}
					public void keyTyped(KeyEvent arg0) {}	
				});

		//recherche par nom de taxon	
		nom_filter.addKeyListener(
				new KeyListener() {
					public void keyPressed(KeyEvent arg0) {}
					public void keyReleased(KeyEvent arg0) {
						String text = ((JTextField)arg0.getSource()).getText();
						if( ! text.isEmpty() )
						{
							code_filter.setText("");
							// tri par la colonne nom taxon
							List<RowSorter.SortKey> keys = new ArrayList<RowSorter.SortKey>();
							keys.add(new RowSorter.SortKey(1, SortOrder.ASCENDING));
							table.getRowSorter().setSortKeys(keys);
							
							for( int i = 0; i < table.getRowCount(); i++ )
								if( ((String)table.getValueAt(i, 1)).toUpperCase().startsWith(text.toUpperCase()))
								{
									self.table.scrollRectToVisible(self.table.getCellRect(i+5, 1, true));
									self.table.getSelectionModel().setSelectionInterval(i, i);
									break;
								}
						}
					}
					public void keyTyped(KeyEvent arg0) {}	
				});

	}

	@SuppressWarnings("serial")
	private void add_table()
	{
		// ajout des champs de recherche par code et par nom puis ajout de la table avec les taxons
		JPanel all = new JPanel();
		JPanel label = new JPanel();
		label.setLayout(new BoxLayout(label, BoxLayout.LINE_AXIS));
		label.setBackground(new Color(192, 197, 199));
		label.setAlignmentX(Component.LEFT_ALIGNMENT);
		all.setLayout(new BoxLayout(all, BoxLayout.PAGE_AXIS));
		all.setBackground(new Color(192, 197, 199));
		recherche_lbl.setAlignmentX(Component.LEFT_ALIGNMENT); 
		label.add(recherche_lbl);

		all.add(Box.createVerticalStrut(5));
		all.add(label);
		code_filter.setMaximumSize(new Dimension(70,20));
		code_filter.setPreferredSize(new Dimension(70,20));
		nom_filter.setMaximumSize(new Dimension(300,20));
		nom_filter.setPreferredSize(new Dimension(300,20));

		JPanel recherche_pane = new JPanel();
		recherche_pane.setLayout(new BoxLayout(recherche_pane, BoxLayout.LINE_AXIS));
		recherche_pane.setAlignmentX(Component.LEFT_ALIGNMENT);
		recherche_pane.setBackground(new Color(192, 197, 199));
		recherche_pane.add(Box.createHorizontalStrut(3));
		recherche_pane.add(code_filter);
		recherche_pane.add(Box.createHorizontalStrut(3));
		recherche_pane.add(nom_filter);
		recherche_pane.add(Box.createHorizontalGlue());

		all.add(Box.createVerticalStrut(5));
		all.add(recherche_pane);
		all.add(Box.createVerticalStrut(3));
		
		
		JScrollPane pane = new JScrollPane(table);

		
		// uniquement necessaire pour la traduction des colonnes de la table
		ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		
		String[] header = { rb.getString("colCodeTaxon"), rb.getString("colNom"), 
							rb.getString("colAuteur"), rb.getString("colInd"), 
							rb.getString("colBioVolCel"), rb.getString("colBioVolIn"), 
							rb.getString("colEmbranchement"), rb.getString("colCreateur"), 
							rb.getString("colFormeCel"), rb.getString("colFormeInd") };

		DefaultTableModel model = new DefaultTableModel() 
		{     			
			// suppression de la possibilit d'diter les cellules
			public boolean isCellEditable(int rowIndex, int columnIndex) {
				return false;
			}
		};

		table.setModel(model);
		((DefaultTableModel)table.getModel()).setColumnIdentifiers(header);
		c_query.fill_table(table);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		table.getTableHeader().setReorderingAllowed(false);
		table.setAutoCreateRowSorter(true);
		
		// ajustement de la largeur des colonnes
		table.getColumnModel().getColumn(0).setPreferredWidth(70);
		table.getColumnModel().getColumn(1).setPreferredWidth(300);
		table.getColumnModel().getColumn(2).setPreferredWidth(225);
		table.getColumnModel().getColumn(3).setPreferredWidth(35);
		table.getColumnModel().getColumn(4).setPreferredWidth(70);
		table.getColumnModel().getColumn(5).setPreferredWidth(70);
		table.getColumnModel().getColumn(6).setPreferredWidth(120);
		table.getColumnModel().getColumn(7).setPreferredWidth(60);
		table.getColumnModel().getColumn(8).setPreferredWidth(130);
		table.getColumnModel().getColumn(9).setPreferredWidth(130);

		// activation de la fonction de tri sur les champs autres que string
		TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(table.getModel());   
		table.setRowSorter(sorter);
		sorter.setComparator(4, new FloatComparator());
		sorter.setComparator(5, new FloatComparator());

		this.add(all, BorderLayout.NORTH);
		this.add(pane, BorderLayout.CENTER);

		
		
		// ouverture de la fentre de modification d'un taxon choisi avec un double clique	
		table.addMouseListener(new MouseAdapter()
		{
			public void mousePressed(MouseEvent e)
			{
				if (e.getClickCount() == 2) {
					String code_choisi = (String)table.getValueAt(table.getSelectedRow(), 0);
					new Modification_Frame(locale, db_link, code_choisi);
					

				}
			}
		});	
		
		
	}

	private void show_frame()
	{
		this.pack();
		this.setSize(table.getWidth() , 800);
		this.setVisible(true);
	}

	// Cration de la comparaison des champs de la table qui sont des entiers
	// pour permmettre le tri correct sur ces champs
	public class IntComparator implements Comparator<String> {

		@Override
		public int compare(String arg0, String arg1) {
			if(Integer.parseInt(arg0) < Integer.parseInt(arg1)) return -1;
			if(Integer.parseInt(arg0) > Integer.parseInt(arg1)) return 1;
			return 0;
		}
	}


	// Cration de la comparaison des champs de la table qui sont des flottants
	// pour permmettre le tri correct sur ces champs
	public class FloatComparator implements Comparator<String> {

		@Override
		public int compare(String o1, String o2) {
			if(Float.parseFloat(o1) < Float.parseFloat(o2)) return -1;
			if(Float.parseFloat(o1) > Float.parseFloat(o2)) return 1;
			return 0;
		}
	}
	
	/**
	 * Mise dans la bonne langue des textes
	 * @param locale
	 */
	public void translate(Locale locale){
		this.locale = locale;
		ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		
		// nom de la fentre
		this.setTitle(rb.getString("titreFenConsultTaxPerso"));	

		fichier.setText(rb.getString("fichier"));
		fermer.setText(rb.getString("fermer"));
		imprimer.setText(rb.getString("imprimer"));
		recherche_lbl.setText(rb.getString("saisieCodeOuNomPerso"));
		
		//ragencement de la position des items de la fentre
		this.pack();
		
	}
	
}
