package C_M_Perso;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Vector;

import utils.Define;

public class Modification_Query {
	private Connection connection = null;
	private Statement query = null;
	private ResultSet rs = null;
	
	@SuppressWarnings("unused")
	private String db_link;
	
	public Modification_Query(String db_link)
	{
		
		this.db_link = db_link;
		
		try
		{
			// connexion  la base de donnes
			Class.forName("com.mysql.jdbc.Driver");
			String url = "jdbc:mysql://" + db_link + "/" + new Define().get_db_name();
			connection = DriverManager.getConnection(url, new Define().get_user(), new Define().get_pass());
			query = connection.createStatement();
		}
		catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Retourne toutes les informations d'un taxon donn
	 * @param code d'un taxon
	 * @return une liste contenant toutes les informations du taxon
	 */
	public String[] get_all_from_liste( String code )
	{
		String[] returned = new String[10];
		ResultSet rs = null;
		try
		{
			if( code.length() == 6 )
				rs = query.executeQuery("SELECT code_taxon, nom_taxon, nom_auteur, "
						+ "indiv_defaut, biovolcel, biovolind, embranchement, createur, "
						+ "(SELECT nom_forme FROM formes WHERE id_forme = liste_taxa_cpt.id_forme_cel) AS cel, "
						+ "(SELECT nom_forme FROM formes WHERE id_forme = liste_taxa_cpt.id_forme_ind) AS ind"
						+ " FROM liste_taxa_cpt WHERE code_taxon = '"+code+"' ORDER BY code_taxon");
			else
				rs = query.executeQuery("SELECT code_taxon, nom_taxon, nom_auteur, "
						+ "indiv_defaut, biovolcel, biovolind, embranchement, createur, "
						+ "(SELECT nom_forme FROM formes WHERE id_forme = liste_taxa_cpt.id_forme_cel) AS cel, "
						+ "(SELECT nom_forme FROM formes WHERE id_forme = liste_taxa_cpt.id_forme_ind) AS ind" +
						" FROM liste_taxa_cpt WHERE nom_taxon = '"+code+"' ORDER BY code_taxon");
			
			
			if( rs.next() )
			{
				// rcupration des donnes retournes par la requte
				returned[0] = rs.getString("code_taxon");
				returned[1] = rs.getString("nom_taxon");
				returned[2] = rs.getString("nom_auteur");
				returned[3] = rs.getString("indiv_defaut");
				returned[4] = ((rs.getFloat("biovolcel")) == 0) ? "" : Float.toString(rs.getFloat("biovolcel"));
				returned[5] = ((rs.getFloat("biovolind")) == 0) ? "" : Float.toString(rs.getFloat("biovolind"));
				returned[6] = rs.getString("embranchement");
				returned[7] = rs.getString("createur");
				returned[8] = rs.getString("cel");
				returned[9] = rs.getString("ind");
			}
			else
				return null;
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Retourne la liste des auteurs dont le nom commence par une chane de caractres donne
	 * @param debut : dbut du nom d'un auteur
	 * @return la liste des auteurs dont le nom commence par debut
	 */
	public String[] get_auteurs_like( String debut )
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();
		
		try
		{
			rs = query.executeQuery("SELECT nom_auteur "
					+ "FROM auteurs "
					+ "WHERE nom_auteur LIKE '%" + debut + "%' "
					+ "UNION "
					+ "SELECT nom_auteur "
					+ "FROM liste_taxa_cpt "
					+ "WHERE nom_auteur LIKE '%" + debut + "%' "
					+ "ORDER BY nom_auteur");
			
			// rcupration des donnes
			while(rs.next())
				tmp.add(rs.getString(1));
			
			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	
	
	/**
	 * Donne la liste des codes de taxons personnels qui commencent par une chane de caractres donne
	 * @param debut : dbut du code d'un taxon
	 * @return tableau contenant les codes qui commencent par dbut
	 */
	public String[] get_codes_like(String debut)
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();
		try
		{
			ResultSet rs = query.executeQuery("SELECT code_taxon "
					+ "FROM liste_taxa_cpt "
					+ "WHERE code_taxon LIKE '" + debut + "%' "
					+ "ORDER BY code_taxon");
			
			while(rs.next())
				tmp.add(rs.getString(1));
			
			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * 
	 * @param codeTax code d'un taxon
	 * @return boolean : indique si le taxon est utilis dans un comptage ou dans un calcul
	 */
	public boolean isUsed(String codeTax){
		boolean used = false;
		Vector<Integer> ids = new Vector<Integer>();

		try {
			// recherche du taxon dans les calculs
			ResultSet rs = query.executeQuery("SELECT id_calcul "
					+ "from calcul_biovol "
					+ "WHERE code_taxon = '"+codeTax+"'");
			
			while(rs.next())
				ids.add(rs.getInt("id_calcul"));
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		try {
			// recherche du taxon dans les comptages
			ResultSet rs = query.executeQuery("SELECT id_cpt "
					+ "from ligne_cpt "
					+ "WHERE code_taxon = '"+codeTax+"'");
			
			while(rs.next())
				ids.add(rs.getInt("id_cpt"));
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		if(!ids.isEmpty())
			used = true;
		
		return used;
	}
	                                   
	/**
	 * Supprime un taxon donn dans la base de donnes
	 * @param codeTax : code du taxon  supprimer
	 */
	public void delTax(String codeTax)
	{
		try
		{
			//suppression d'un taxon cpt
			query.executeUpdate("DELETE FROM liste_taxa_cpt WHERE code_taxon = '"+codeTax+"'");
		}
		catch(SQLException e){ e.printStackTrace(); };
	}
	
	
	/**
	 * 
	 * @param debut
	 * @return tableau contenant les noms des taxons dont le nom commence par debut
	 */
	public String[] get_noms_like(String debut)
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();
		try
		{
			ResultSet rs = query.executeQuery("SELECT nom_taxon "
					+ "FROM liste_taxa_cpt "
					+ "WHERE nom_taxon LIKE '" + debut + "%' " 
					+ "ORDER BY nom_taxon");
			
			while(rs.next())
				tmp.add(rs.getString(1));
			
			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Met  jour un taxon donn avec ses nouvelles informations
	 * @param code_taxon
	 * @param nom_taxon
	 * @param nom_auteur
	 * @param indiv
	 * @param biovolcel
	 * @param biovolind
	 * @param embr
	 * @param createur
	 * @param formeCel
	 * @param formeInd
	 */
	public void update_taxon_cpt(String code_taxon, String nom_taxon, String nom_auteur, String indiv, float biovolcel,
			float biovolind, String embr, String createur, String formeCel, String formeInd)
	{
		Object idCel = null, idInd = null;
		
		if(formeCel.compareTo("") !=0 ){
		try
		{
			rs = query.executeQuery("SELECT id_forme AS id_forme_cel "
					+ "FROM formes "
					+ "WHERE nom_forme ='"+formeCel+"'");
			
			if( rs.next() )
			{	
				idCel = rs.getInt("id_forme_cel");	
			}
			else
			{
				idCel = null;
			}
		}
		catch(SQLException e) { e.printStackTrace(); }
		}
		
		if(formeInd.compareTo("") !=0 ){
		try
		{
			rs = query.executeQuery("SELECT id_forme AS id_forme_ind "
					+ "FROM formes "
					+ "WHERE nom_forme ='"+formeInd+"'");
			if( rs.next() )
			{	
				idInd = rs.getInt("id_forme_ind");	
			}
			else
			{
				idInd = null;
			}
		}
		catch(SQLException e) { e.printStackTrace(); }
		}
		
		try
		{
			query.executeUpdate("UPDATE liste_taxa_cpt SET nom_taxon = '" +nom_taxon.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + 
					"', nom_auteur = '" + nom_auteur.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "', indiv_defaut = '" + indiv +
					"', biovolcel = " +	((biovolcel != 0) ? biovolcel : "NULL") + ", biovolind = " + ((biovolind != 0) ? biovolind : "NULL") +
					", embranchement = " + (!embr.equals("") ? "'" + embr.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") +"'" : "NULL") + ", " +
					"createur = '" + createur.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "', " +
							"id_forme_cel = "+idCel+", id_forme_ind = "+idInd+" WHERE code_taxon = '" +
					code_taxon + "'");
		}
		catch(SQLException e) { e.printStackTrace(); }
	}
	
	
	
	/**
	 * 
	 * @param debut_embr
	 * @return Une liste de String correspondant aux embranchements distincts qui commencent
	 * par les lettres que l'utilisateur  saisies dans le champ embranchement.
	 * 
	 */
	public String[] get_embranchement_designes(String debut_embr){
		String[] liste_embr_designes = null;
		ArrayList<String> tmp = new ArrayList<String>();
		
		try{
			
			rs = query.executeQuery(
					"SELECT DISTINCT embranchement " +
					"FROM taxinomie " + 
					"WHERE embranchement LIKE '" + debut_embr.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "%' " +
					"ORDER BY embranchement");
			
			while(rs.next()){
				if (rs.getString(1) != null)
					tmp.add(rs.getString(1));
			}
			
			if (tmp.size()>0){
				
				liste_embr_designes = new String[tmp.size()];
				
				for(int i =0; i<tmp.size(); ++i)
					liste_embr_designes[i] = tmp.get(i);
			}
			
		}
		catch (SQLException e) { e.printStackTrace(); }
			
		return liste_embr_designes;
	}

}
