package C_R_Valides;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.RowSorter;
import javax.swing.SortOrder;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import C_M_Perso.Consultation_Perso_Frame;
import Calcul_Biovolume.Calcul_Biovolume_Frame;
import Calcul_Biovolume.Ouvrir_Calcul;
import Comptage.Comptage_Frame;
import Menus.Gestion_IPLAC_Frame;


public class Consultation_Frame extends JFrame {

	private static final long serialVersionUID = 1L;
	private Locale locale;
	private String db_link;
	
	//cration du chemin vers dossiers docs
	private String pathDocs = new String("Docs");

	private Comptage_Frame cf = null;
	private Calcul_Biovolume_Frame cbf = null;
	private Ouvrir_Calcul oc = null;
	
	private JMenuBar menu_bar = new JMenuBar();
	// menu menu
	private JMenu menu = new JMenu();
	private JMenuItem comptage = new JMenuItem();
	private JMenuItem iplac = new JMenuItem();
	private JMenuItem fermer = new JMenuItem();
	
	// menu calcul de biovolume
	private JMenu menu_calcul = new JMenu();
	private JMenuItem newCalc = new JMenuItem();
	private JMenuItem openCalc = new JMenuItem();
	
	// menu taxon
	private JMenu taxons = new JMenu();
	private JMenuItem recherchTax = new JMenuItem();
	private JMenu taxon_perso = new JMenu();
	private JMenuItem consult_tax_perso = new JMenuItem();
	
	// menu ?
	private JMenu apropos = new JMenu("?");
	private JMenu version = new JMenu();
	private JMenuItem date = new JMenuItem();
	private JMenuItem notice = new JMenuItem();
	private JMenuItem biblio = new JMenuItem();
	
	private JTextField cdsandre_filter = new JTextField();
	private JTextField code_filter = new JTextField();
	private JTextField nom_filter = new JTextField();

	private JLabel recherche_lbl = new JLabel();

	private Consultation_Frame self = null;

	private JTable table = new JTable();

	private Consultation_Query c_query;
	
	private boolean ocOuvert = false;


	public Consultation_Frame(Locale locale, String db_link)
	{
		super();
		self = this;
		this.locale = locale;
		this.db_link = db_link;

		c_query = new Consultation_Query(db_link);

		this.setLayout(new BorderLayout());
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

		add_menu();
		add_rech_table();

		this.translate(locale);

		add_handlers();
		show_frame();	
	}


	// boolen pour ne pas autoriser la fermeture de l'application si l'cran comptage
	// est ouvert avec un comptage 
	public boolean get_closable()
	{
		return ((cf != null && cf.get_id_cpt() != 0) ? false : true);
	}

	// boolen pour ne pas autoriser la fermeture de l'application si l'cran calcul biovol
	// est ouvert avec un calcul nouveau ou r-ouvert
	public boolean get_closableCalc()
	{
		return (( cbf != null && (cbf.get_id_calcul() != 0 || cbf.new_calc())
				|| (ocOuvert == true && ! oc.get_closableCalc())) ? false : true);
	}
	
	private void add_menu()
	{
		menu.add(comptage);
		menu.add(iplac);		
		menu.add(fermer);
		menu_bar.add(menu);
		
		menu_calcul.add(newCalc);
		menu_calcul.add(openCalc);
		menu_bar.add(menu_calcul);
		
		taxons.add(recherchTax);
		menu_bar.add(taxons);
		
		taxon_perso.add(consult_tax_perso);
		menu_bar.add(taxon_perso);
		
		date.setText(c_query.get_last_date());
		version.add(date);
		apropos.add(version);
		apropos.add(notice);
		apropos.add(biblio);
		menu_bar.add(apropos);

		this.setJMenuBar(menu_bar);
	}

	private void add_handlers()
	{
		// traduction des botes de dialogue
		final ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);

		comptage.addActionListener(
				new ActionListener()  {
					public void actionPerformed(ActionEvent e) {
						cf = new Comptage_Frame(locale, db_link);
					}});

		iplac.addActionListener(
				new ActionListener()  {
					public void actionPerformed(ActionEvent e) {
						new Gestion_IPLAC_Frame(locale, db_link, "3", "01/05", "31/10", true, false);
					}});

		fermer.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						self.dispose();
					}});
		
		// ouverture de la fenetre nouveau calcul de biovolume
		newCalc.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				cbf = new Calcul_Biovolume_Frame(locale, db_link);				
			}
		});

		// ouverture de la fenetre d'un calcul de biovolume existant
		openCalc.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				oc = new Ouvrir_Calcul(null, false, false, locale, db_link);
				ocOuvert =true;
			}
		});
		
		recherchTax.addActionListener(
				new ActionListener()  {
					public void actionPerformed(ActionEvent e) {
						new Recherche_Frame(locale, db_link);
					}});

		consult_tax_perso.addActionListener(
				new ActionListener()  {
					public void actionPerformed(ActionEvent e) {
						new Consultation_Perso_Frame(locale, db_link);
					}});

		notice.addActionListener(
				new ActionListener() {
					@SuppressWarnings("static-access")
					public void actionPerformed(ActionEvent e) {

						File bibli = new File(pathDocs + File.separator + "notice.pdf");

						if(!bibli.exists())
							bibli = new File(pathDocs + File.separator + "notice.pdf");

						System.out.println(bibli.getAbsolutePath());

						if(Desktop.isDesktopSupported()) 
						{
							if(Desktop.getDesktop().isSupported(java.awt.Desktop.Action.OPEN))
							{
								try { java.awt.Desktop.getDesktop().open(bibli);} 
								catch (IOException ex) { ex.printStackTrace(); }
							} 
							else 
								new JOptionPane().showMessageDialog(self, rb.getString("mssgFonctionNonSupportee"), 
										rb.getString("titreBoxFonctionNonSupp"), JOptionPane.ERROR_MESSAGE);
						}
						else
							new JOptionPane().showMessageDialog(self, rb.getString("mssgFonctionNonSupportee"), 
									rb.getString("titreBoxFonctionNonSupp"), JOptionPane.ERROR_MESSAGE);
					}});
		
		biblio.addActionListener(
				new ActionListener() {
					@SuppressWarnings("static-access")
					public void actionPerformed(ActionEvent e) {

						File bibli = new File(pathDocs + File.separator + "biblio.pdf");

						if(!bibli.exists())
							bibli = new File(pathDocs + File.separator + "biblio.pdf");

						System.out.println(bibli.getAbsolutePath());

						if(Desktop.isDesktopSupported()) 
						{
							if(Desktop.getDesktop().isSupported(java.awt.Desktop.Action.OPEN))
							{
								try { java.awt.Desktop.getDesktop().open(bibli);} 
								catch (IOException ex) { ex.printStackTrace(); }
							} 
							else 
								new JOptionPane().showMessageDialog(self, rb.getString("mssgFonctionNonSupportee"), 
										rb.getString("titreBoxFonctionNonSupp"), JOptionPane.ERROR_MESSAGE);
						}
						else
							new JOptionPane().showMessageDialog(self, rb.getString("mssgFonctionNonSupportee"), 
									rb.getString("titreBoxFonctionNonSupp"), JOptionPane.ERROR_MESSAGE);
					}});

		//recherche par code Sandre	
		cdsandre_filter.addKeyListener(
				new KeyListener() {
					public void keyPressed(KeyEvent arg0) {}
					public void keyReleased(KeyEvent arg0) {
						String text = ((JTextField)arg0.getSource()).getText();
						if( ! text.isEmpty() )
						{
							code_filter.setText("");
							nom_filter.setText("");
							// tri par la colonne code Sandre
							List<RowSorter.SortKey> keys = new ArrayList<RowSorter.SortKey>();
							keys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
							table.getRowSorter().setSortKeys(keys);
							
							new IntComparator();
							for( int i = 0; i < table.getRowCount(); i++ )
								if((String)table.getValueAt(i, 0)!=null)
								{
								if(	((String)table.getValueAt(i, 0)).startsWith(text))
								{
									self.table.scrollRectToVisible(self.table.getCellRect(i+5, 0, true));
									self.table.getSelectionModel().setSelectionInterval(i, i);
									break;
								}
						}
						}
					}
						public void keyTyped(KeyEvent arg0) {}	
					});

		//recherche par code 6 lettres	
		code_filter.addKeyListener(
				new KeyListener() {
					public void keyPressed(KeyEvent arg0) {}
					public void keyReleased(KeyEvent arg0) {
						String text = ((JTextField)arg0.getSource()).getText();
						if( ! text.isEmpty() )
						{
							cdsandre_filter.setText("");
							nom_filter.setText("");
							// tri par la colonne code taxon
							List<RowSorter.SortKey> keys = new ArrayList<RowSorter.SortKey>();
							keys.add(new RowSorter.SortKey(1, SortOrder.ASCENDING));
							table.getRowSorter().setSortKeys(keys);
							
							for( int i = 0; i < table.getRowCount(); i++ )
								if( ((String)table.getValueAt(i, 1)).toUpperCase().startsWith(text.toUpperCase()))
								{
									self.table.scrollRectToVisible(self.table.getCellRect(i+5, 1, true));
									self.table.getSelectionModel().setSelectionInterval(i, i);
									break;
								}
						}
					}
					public void keyTyped(KeyEvent arg0) {}	
				});

		//recherche par nom de taxon	
		nom_filter.addKeyListener(
				new KeyListener() {
					public void keyPressed(KeyEvent arg0) {}
					public void keyReleased(KeyEvent arg0) {
						String text = ((JTextField)arg0.getSource()).getText();
						if( ! text.isEmpty() )
						{
							code_filter.setText("");
							cdsandre_filter.setText("");
							// tri par la colonne nom taxon
							List<RowSorter.SortKey> keys = new ArrayList<RowSorter.SortKey>();
							keys.add(new RowSorter.SortKey(2, SortOrder.ASCENDING));
							table.getRowSorter().setSortKeys(keys);
							
							for( int i = 0; i < table.getRowCount(); i++ )
								if( ((String)table.getValueAt(i, 2)).toUpperCase().startsWith(text.toUpperCase()))
								{
									self.table.scrollRectToVisible(self.table.getCellRect(i+5, 2, true));
									self.table.getSelectionModel().setSelectionInterval(i, i);
									break;
								}
						}
					}
					public void keyTyped(KeyEvent arg0) {}	
				});

				}

				//ajout des champs et du bandeau de recherche d'un taxon ou d'un code puis de la table
				private void add_rech_table() {
					JPanel all = new JPanel();
					JPanel label = new JPanel();
					label.setLayout(new BoxLayout(label, BoxLayout.LINE_AXIS));
					label.setBackground(new Color(192, 197, 199));
					label.setAlignmentX(Component.LEFT_ALIGNMENT);
					all.setLayout(new BoxLayout(all, BoxLayout.PAGE_AXIS));
					all.setBackground(new Color(192, 197, 199));
					recherche_lbl.setAlignmentX(Component.LEFT_ALIGNMENT); 
					label.add(recherche_lbl);

					all.add(Box.createVerticalStrut(5));
					all.add(label);
					cdsandre_filter.setMaximumSize(new Dimension(70,20));
					cdsandre_filter.setPreferredSize(new Dimension(70,20));
					code_filter.setMaximumSize(new Dimension(70,20));
					code_filter.setPreferredSize(new Dimension(70,20));
					nom_filter.setMaximumSize(new Dimension(300,20));
					nom_filter.setPreferredSize(new Dimension(300,20));

					JPanel recherche_pane = new JPanel();
					recherche_pane.setLayout(new BoxLayout(recherche_pane, BoxLayout.LINE_AXIS));
					recherche_pane.setAlignmentX(Component.LEFT_ALIGNMENT);
					recherche_pane.setBackground(new Color(192, 197, 199));
					recherche_pane.add(Box.createHorizontalStrut(2));
					recherche_pane.add(cdsandre_filter);
					recherche_pane.add(Box.createHorizontalStrut(3));
					recherche_pane.add(code_filter);
					recherche_pane.add(Box.createHorizontalStrut(3));
					recherche_pane.add(nom_filter);
					recherche_pane.add(Box.createHorizontalGlue());

					all.add(Box.createVerticalStrut(5));
					all.add(recherche_pane);
					all.add(Box.createVerticalStrut(3));

					//cration du panneau de la table
					JScrollPane pane = new JScrollPane(table);

					// traduction des libells des colonnes de la table
					ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);

					// noms des colonnes de la table
					String[] header = { rb.getString("colCodeSandre"), rb.getString("colCodeTaxon"), 
							rb.getString("colNom"), rb.getString("colAuteur"), 
							rb.getString("colInd"), rb.getString("colBioVolCel"), 
							rb.getString("colBioVolIn"), rb.getString("colCodeRef"), 
							rb.getString("colFormeCel"), rb.getString("colFormeInd"), 
							rb.getString("colCoteSpec"), rb.getString("colCoefSteno"), 
							rb.getString("colTrophie"), rb.getString("colSladS"), 
							rb.getString("colSladV"), rb.getString("colRegne"), 
							rb.getString("colSousRegne"), rb.getString("colEmbranchement"), 
							rb.getString("colClasse"), rb.getString("colOrdre"), 
							rb.getString("colSousOrdre"), rb.getString("colFamille"), 
							rb.getString("colGrpAlgal"), rb.getString("colCodeO"), 
							rb.getString("colDateMaJ"), rb.getString("validite") };


					DefaultTableModel model = new DefaultTableModel()
					{
						/**
						 * 
						 */
						private static final long serialVersionUID = 1L;

						//suppression de la possibilit d'diter les cellules
						public boolean isCellEditable(int rowIndex, int columnIndex) {
							return false;
						}
					};


					table.setModel(model);
					((DefaultTableModel)table.getModel()).setColumnIdentifiers(header);;
					c_query.fill_table(table);
					table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
					table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
					table.getTableHeader().setReorderingAllowed(false);
					table.setAutoCreateRowSorter(true);

					// ajustement de la largeur des colonnes
					table.getColumnModel().getColumn(1).setPreferredWidth(70);
					table.getColumnModel().getColumn(2).setPreferredWidth(300);
					table.getColumnModel().getColumn(3).setPreferredWidth(270);
					table.getColumnModel().getColumn(4).setPreferredWidth(40);
					table.getColumnModel().getColumn(7).setPreferredWidth(70);
					table.getColumnModel().getColumn(8).setPreferredWidth(100);
					table.getColumnModel().getColumn(9).setPreferredWidth(100);
					table.getColumnModel().getColumn(15).setPreferredWidth(90);
					table.getColumnModel().getColumn(16).setPreferredWidth(120);
					table.getColumnModel().getColumn(17).setPreferredWidth(150);
					table.getColumnModel().getColumn(18).setPreferredWidth(150);
					table.getColumnModel().getColumn(19).setPreferredWidth(150);
					table.getColumnModel().getColumn(20).setPreferredWidth(120);
					table.getColumnModel().getColumn(21).setPreferredWidth(150);
					table.getColumnModel().getColumn(22).setPreferredWidth(60);
					table.getColumnModel().getColumn(23).setPreferredWidth(60);
					table.getColumnModel().getColumn(25).setPreferredWidth(50);
					
					// activation de la fonction de tri sur les champs autres que string
					TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(table.getModel());   
					table.setRowSorter(sorter);
					sorter.setComparator(0, new IntComparator());
					sorter.setComparator(5, new FloatComparator());
					sorter.setComparator(6, new FloatComparator());
					sorter.setComparator(10, new FloatComparator());
					sorter.setComparator(11, new FloatComparator());
					sorter.setComparator(12, new FloatComparator());
					sorter.setComparator(13, new FloatComparator());
					sorter.setComparator(14, new IntComparator());
					sorter.setComparator(22, new IntComparator());

					this.add(all, BorderLayout.NORTH);
					this.add(pane, BorderLayout.CENTER);

					// ouverture de la fentre de consultation d'un taxon valide choisi avec un double clique	
					table.addMouseListener(new MouseAdapter()
					{
						public void mousePressed(MouseEvent e)
						{
							if (e.getClickCount() == 2) {
								String code_choisi = (String)table.getValueAt(table.getSelectedRow(), 1);
								new Recherche_Frame(locale, db_link, code_choisi);

							}
						}
					});	
				}



				private void show_frame()
				{
					this.pack();
					self.setSize(new Dimension(1280, 1024));
					this.setVisible(true);
				}	


				// Cration de la comparaison des champs de la table qui sont des entiers
				// pour permmettre le tri correct sur ces champs
				public class IntComparator implements Comparator<String> {

					@Override
					public int compare(String arg0, String arg1) {
						if(Integer.parseInt(arg0) < Integer.parseInt(arg1)) return -1;
						if(Integer.parseInt(arg0) > Integer.parseInt(arg1)) return 1;
						return 0;
					}
				}


				// Cration de la comparaison des champs de la table qui sont des flottants
				// pour permmettre le tri correct sur ces champs
				public class FloatComparator implements Comparator<String> {

					@Override
					public int compare(String o1, String o2) {
						if(Float.parseFloat(o1) < Float.parseFloat(o2)) return -1;
						if(Float.parseFloat(o1) > Float.parseFloat(o2)) return 1;
						return 0;
					}
				}

				/**
				 * Traduction dans la bonne langue des textes de la page
				 * @param locale
				 */
				public void translate(Locale locale){
					this.locale = locale;
					ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);

					// nom de la fentre
					this.setTitle(rb.getString("titreFenConsultTaxValid"));	

					menu.setText(rb.getString("menu"));
					iplac.setText(rb.getString("calculIPLAC"));
					comptage.setText(rb.getString("fenetreComptage"));
					fermer.setText(rb.getString("fermer"));
					menu_calcul.setText(rb.getString("calculDeBioVol"));
					newCalc.setText(rb.getString("nouveauCalcul"));
					openCalc.setText(rb.getString("ouvrirCalcul"));
					taxons.setText(rb.getString("taxonsValides"));
					recherchTax.setText(rb.getString("rechercheTaxon"));
					notice.setText(rb.getString("notice"));
					biblio.setText(rb.getString("bibliographie"));
					version.setText(rb.getString("versionDu"));
					taxon_perso.setText(rb.getString("taxonsPersonnels"));
					consult_tax_perso.setText(rb.getString("consulterEtModifier"));
					recherche_lbl.setText(rb.getString("saisieCodeOuNom"));

					//ragencement de la position des items de la fentre
					this.pack();

				}

	}