package Calcul_Biovolume;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Vector;

import utils.Define;

public class Calcul_Biovolume_Query {

	private Connection connection = null;
	private Statement query = null;
	private ResultSet rs = null;
	private ResultSet rss = null;
	
	@SuppressWarnings("unused")
	private String db_link;
	
	public Calcul_Biovolume_Query(String db_link)
	{
		this.db_link = db_link;
		
		try
		{
			// connexion  la base de donnes
			Class.forName("com.mysql.jdbc.Driver");
			String url = "jdbc:mysql://" + db_link + "/" + new Define().get_db_name();
			connection = DriverManager.getConnection(url, new Define().get_user(), new Define().get_pass());
			query = connection.createStatement();
		}
		catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Retourne toutes les infos de tous les calculs pour la fentre d'ouverture d'un calcul
	 * @return une matrice de donnes
	 */
	public Object[][] get_data()
	{
		Object[][] returned = null;
		ArrayList<Object> data = new ArrayList<Object>();
		
		try 
		{
			rs = query.executeQuery("SELECT calcul_biovol.code_taxon, "
					+ "(SELECT nom_taxon FROM liste_taxa " +
					"WHERE liste_taxa.code_taxon = calcul_biovol.code_taxon) AS nom, " +
					"(SELECT nom_taxon FROM liste_taxa_cpt WHERE liste_taxa_cpt.code_taxon = calcul_biovol.code_taxon)" +
					" AS nom_cpt, type, biovol_moy, date, nom_operateur, id_calcul FROM calcul_biovol ORDER BY nom, nom_cpt");
			
			while( rs.next() )
			{
				data.add(rs.getString("code_taxon"));
				data.add(rs.getString("nom"));
				if(data.get(data.size()-1) == null)
					data.set(data.size()-1, rs.getString("nom_cpt"));
				data.add(rs.getString("type"));
				data.add(rs.getString("biovol_moy"));
				data.add(rs.getString("date"));
				data.add(rs.getString("nom_operateur"));
				data.add(rs.getString("id_calcul"));
			}
			
			returned = new Object[data.size()/7][7];
			
			for( int i = 0; i < data.size(); i++)
				returned[i/7][i%7] = (Object)data.get(i);
		} 
		catch (SQLException e) {
			e.printStackTrace();
		}
		
		return returned;
	}
	
	/**
	 * Retourne les donnes du cartouche d'un comptage
	 * @param id_cpt id du comptage
	 * @return un tableau de donnes
	 */
	public String[] get_all_up_for_open( int id_calcul )
	{
		String[] returned = new String[16];
		int id;
		String com;
		
		try
		{
			rs = query.executeQuery("SELECT code_taxon, date, ref_etude, nom_operateur, commentaire, " +
					"biovol_moy, biovol_max, biovol_min, id_forme_choisie, type, nb_cellule_moy, "
					+ "id_station "
					+ "FROM calcul_biovol "
					+ "WHERE id_calcul = '"+id_calcul+"'");
			
			rs.next();
			
			returned[0] = rs.getString("code_taxon");
			returned[1] = rs.getString("date");
			returned[2] = rs.getString("ref_etude");
			returned[3] = rs.getString("nom_operateur");
			com = rs.getString("commentaire");
			if( com == null ) 
				com = "";
			returned[4] = com;
			returned[5] = Float.toString(rs.getFloat("biovol_moy"));
			returned[6] = Float.toString(rs.getFloat("biovol_max"));
			returned[7] = Float.toString(rs.getFloat("biovol_min"));
			returned[8] = Integer.toString(rs.getInt("id_forme_choisie"));
			returned[9] = rs.getString("type");
			
			if(rs.getObject("nb_cellule_moy") != null)
				returned[11] = Float.toString(rs.getFloat("nb_cellule_moy"));
			else
				returned[11] = "N/A";
			
			if(rs.getObject("id_station") != null){
				id = rs.getInt("id_station");

				rss = query.executeQuery("SELECT nom_station, id_lieu, milieu "
						+ "FROM stations "
						+ "WHERE id_station = '"+ id +"'");
				rss.next();
				returned[13] = rss.getString("nom_station");
				returned[14] = rss.getString("milieu");
				id = rss.getInt("id_lieu");
			
				rss = query.executeQuery("SELECT nom_lieu "
						+ "FROM lieux "
						+ "WHERE id_lieu = '"+ id +"'");
				rss.next();
			
				returned[12] = rss.getString("nom_lieu");
				returned[15] = "has_station";
			}
			else
				returned[15] = "no_station";
			
		}
		catch(SQLException e){ e.printStackTrace(); }
		return returned;
	}
	
	/**
	 * 
	 * @param id_calcul
	 * @return nombre de lignes d'un calcul
	 */
	public int getNbLines(int id_calcul){
		int nb_lines = 0;
		
		try{
		rs = query.executeQuery("SELECT id_ligne "
				+ "FROM ligne_calcul "
				+ "WHERE id_calcul = '"+id_calcul+"'");
		
		while (rs.next()) 
			nb_lines++;

		}
		
		catch(SQLException e){ e.printStackTrace(); }
		
		return nb_lines;
	}
	
	/**
	 * 
	 * @param id_calcul
	 * @param nb_lines
	 * @param columns
	 * @return les lignes d'un calcul donn
	 */
	public Object[][] getLineDatas(int id_calcul, int nb_lines, Vector<String> columns){
		Object[][] datas = new Object[nb_lines][columns.size()];

		try{
			rs = query.executeQuery("SELECT id_ligne FROM ligne_calcul WHERE id_calcul = '"+id_calcul+"'");
			for (int i = 0; i < nb_lines; i++){
				rs.next();
				datas[i][0] = Integer.parseInt(rs.getString("id_ligne"));
			}
			
			rs = query.executeQuery("SELECT biovol_objet FROM ligne_calcul WHERE id_calcul = '"+id_calcul+"'");
			for (int i = 0; i < nb_lines; i++){
				rs.next();
				datas[i][1] = Float.parseFloat(rs.getString("biovol_objet"));
			}
			
			for(int j = 2; j < columns.size()-2; j++){
					rs = query.executeQuery("SELECT "+columns.elementAt(j)+" FROM ligne_calcul WHERE id_calcul = '"+id_calcul+"'");
					for(int i =0; i < nb_lines ; i++){
						rs.next();
						datas[i][j] = rs.getString(columns.elementAt(j));
					}
			}
			
		}
		
		catch(SQLException e){ e.printStackTrace(); }
		
		return datas;
	}
	

	
	/**
	 * Ajoute un calcul dans la base et cre l'chantillon associ
	 * @param code_taxon
	 * @param taximoniste
	 * @param type
	 * @param id_forme
	 * @param site
	 * @param station
	 * @param date
	 * @param ref_ech
	 * @return l'identifiant du calcul qui a t ajout
	 */
	public int add_calcul(String code_taxon, String taximoniste, String type, int id_forme, String site, String station, String date, String ref_ech)
	{
		int id_station, id_calcul = 0;
		try
		{
			if(site != null && station != null){
				rs = query.executeQuery("SELECT id_station FROM stations "
						+ "WHERE nom_station = '" + station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") 
						+ "' AND "
						+ " id_lieu = (SELECT id_lieu FROM lieux WHERE nom_lieu = '" + site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
						+ "')");
			
				rs.next();
				id_station = rs.getInt(1);
				
				query.executeUpdate("INSERT INTO calcul_biovol "
						+ "VALUES (NULL, '"+ code_taxon +"', '"+type+"','" +id_station+ "','" 
						+date+"','" +ref_ech+ "','" +taximoniste+ "', NULL, NULL, NULL, NULL, NULL, '"
						+id_forme+"')", Statement.RETURN_GENERATED_KEYS); 
				
				rs = query.getGeneratedKeys();
				rs.next();
				id_calcul = rs.getInt(1);
			}
			else{
				query.executeUpdate("INSERT INTO calcul_biovol "
						+ "VALUES (NULL, '"+ code_taxon +"', '"+type+"',null,'" +date+"','" 
						+ref_ech+ "','" +taximoniste+ "', NULL, NULL, NULL, NULL, NULL,'"
						+id_forme+"')", Statement.RETURN_GENERATED_KEYS); 
				
				rs = query.getGeneratedKeys();
				rs.next();
				id_calcul = rs.getInt(1);
			}
			
		}
		catch( SQLException e) { e.printStackTrace(); }
		
		return id_calcul;
	}
	
	
	/**
	 * Met  jour les biovol moy min et max d'un calcul
	 * @param id_calcul
	 * @param biovol_moy
	 * @param biovol_min
	 * @param biovol_max
	 * @param nb_cellule_moy
	 * @param com
	 * @param isCol
	 */
	public void updateCalcul(int id_calcul, float biovol_moy, float biovol_min, float biovol_max, String nb_cellule_moy, String com, boolean isCol)
	{
		try	{
			query.executeUpdate("UPDATE calcul_biovol "
					+ "SET biovol_moy= '" + biovol_moy 
					+ "' WHERE id_calcul = '" + id_calcul + "'");
			
			query.executeUpdate("UPDATE calcul_biovol "
					+ "SET biovol_min= '" + biovol_min 
					+ "' WHERE id_calcul = '" + id_calcul + "'");
			
			query.executeUpdate("UPDATE calcul_biovol "
					+ "SET biovol_max= '" + biovol_max 
					+ "' WHERE id_calcul = '" + id_calcul + "'");
			
			if(isCol)
				query.executeUpdate("UPDATE calcul_biovol "
						+ "SET nb_cellule_moy= '" + Float.parseFloat(nb_cellule_moy) 
						+ "' WHERE id_calcul = '" + id_calcul + "'");
			else
				query.executeUpdate("UPDATE calcul_biovol "
						+ "SET nb_cellule_moy= NULL");
			
			query.executeUpdate("UPDATE calcul_biovol "
					+ "SET commentaire= '" + com 
					+ "' WHERE id_calcul = '" + id_calcul + "'");
		}
		catch( SQLException e) { e.printStackTrace(); }
	}

	/**
	 * Met  jour le cartouche gnral : le code taxon, taxonomiste, obj mesur, id_forme, commentaire 
	 * id_station, date et ref chant d'un calcul
	 * @param id_calcul
	 * @param code_taxon
	 * @param taxonomiste
	 * @param obj_mesure
	 * @param id_forme
	 * @param com
	 * @param id_station
	 * @param date
	 * @param ref_echant
	 */
	public int updateCalc(int id_calcul, String code_taxon, String taxinomiste, String obj_mesure,
			int id_forme, String com, String site, String station, String date, String ref_echant)
	{
		int id_station = 0;
			try	{				
			query.executeUpdate("UPDATE calcul_biovol "
					+ "SET code_taxon = '" + code_taxon 
					+ "' WHERE id_calcul = '" + id_calcul + "'");
			
			query.executeUpdate("UPDATE calcul_biovol "
					+ "SET nom_operateur = '" + taxinomiste 
					+ "' WHERE id_calcul = '" + id_calcul + "'");
			
			query.executeUpdate("UPDATE calcul_biovol "
					+ "SET type = '" + obj_mesure 
					+ "' WHERE id_calcul = '" + id_calcul + "'");
		
			query.executeUpdate("UPDATE calcul_biovol "
					+ "SET id_forme_choisie = '" + id_forme 
					+ "' WHERE id_calcul = '" + id_calcul + "'");
			
			query.executeUpdate("UPDATE calcul_biovol "
					+ "SET commentaire= '" + com 
					+ "' WHERE id_calcul = '" + id_calcul + "'");
			
			if(site != null && station != null){
				rs = query.executeQuery("SELECT id_station FROM stations "
						+ "WHERE nom_station = '" + station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") 
						+ "' AND "
						+ " id_lieu = (SELECT id_lieu FROM lieux WHERE nom_lieu = '" + site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
						+ "')");
			
				rs.next();
				id_station = rs.getInt(1);
				
				query.executeUpdate("UPDATE calcul_biovol "
						+ "SET id_station = '" + id_station 
						+ "' WHERE id_calcul = '" + id_calcul + "'");
				
				query.executeUpdate("UPDATE calcul_biovol "
						+ "SET date = '" + date 
						+ "' WHERE id_calcul = '" + id_calcul + "'");
				
				query.executeUpdate("UPDATE calcul_biovol "
						+ "SET ref_etude = '" + ref_echant 
						+ "' WHERE id_calcul = '" + id_calcul + "'");
			}

		}
		catch( SQLException e) { e.printStackTrace(); }
			return id_calcul;
	}
	
	/**
	 * Ajoute une ligne dans un calcul
	 * @param id_cpt id du comptage
	 * @param id_line id de la ligne 
	 */
	public void add_line(int id_calcul, int id_line)
	{
		try
		{
			query.executeUpdate("INSERT INTO ligne_calcul "
					+ "VALUES ('" + id_line + "', '" + id_calcul 
					+ "', NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)");
		}
		catch(SQLException e){ e.printStackTrace(); };
	}
	
	/**
	 * Supprime un calcul donn
	 * @param id_calcul du calcul  supprimer
	 */
	public void delCalcul(int id_calcul)
	{
		try
		{
			//suppression d'un calcul, et des lignes associes
			query.executeUpdate("DELETE FROM calcul_biovol WHERE id_calcul = '"+id_calcul+"'");
			query.executeUpdate("DELETE FROM ligne_calcul WHERE id_calcul = '"+id_calcul+"'");
		}
		catch(SQLException e){ e.printStackTrace(); };
	}
	
	
	/**
	 * ajoute des lignes dans un calcul
	 * @param id_calcul
	 * @param data
	 * @param columns
	 * @param nb_lines
	 */
	@SuppressWarnings("rawtypes")
	public void add_lines(int id_calcul, Vector data, Vector<String> columns, int nb_lines){
		System.out.println("nbligne"+nb_lines);
		columns.setElementAt("biovol_objet", 1);
		try 
		{
			for(int i = 0; i < nb_lines-1; i++){
				System.out.println(Integer.parseInt(((Vector)data.elementAt(i)).elementAt(0).toString()));
				query.executeUpdate("INSERT INTO ligne_calcul VALUES ('" + Integer.parseInt(((Vector)data.elementAt(i)).elementAt(0).toString()) 
						+ "', '"+id_calcul+"', NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)");
				
				for(int j = 1; j < columns.size()-2; j++){
					if(!(((Vector)data.elementAt(i)).elementAt(j).toString()).isEmpty())
						query.executeUpdate("UPDATE ligne_calcul "
								+ "SET "+columns.elementAt(j)+" = '"+ Float.parseFloat(((Vector)data.elementAt(i)).elementAt(j).toString()) 
								+ "' WHERE id_ligne = '" + Integer.parseInt(((Vector)data.elementAt(i)).elementAt(0).toString()) 
								+ "' AND id_calcul = '"+id_calcul+"'");
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Met  jour les lignes d'un calcul donn
	 * @param id_calcul
	 * @param data
	 * @param columns
	 * @param nb_lines
	 */
	@SuppressWarnings("rawtypes")
	public void update_lines(int id_calcul, Vector data, Vector<String> columns, int nb_lines)
	{
		columns.setElementAt("biovol_objet", 1);
		try 
		{
			query.executeUpdate("DELETE FROM ligne_calcul "
					+ "WHERE id_calcul = '"+ id_calcul+"'");
			
			for(int i = 0; i < nb_lines-1; i++){
				query.executeUpdate("INSERT INTO ligne_calcul "
						+ "VALUES ('" + Integer.parseInt(((Vector)data.elementAt(i)).elementAt(0).toString()) 
						+ "', '"+id_calcul+"', NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)");
				
				for(int j = 1; j < columns.size()-2; j++){
					if(!(((Vector)data.elementAt(i)).elementAt(j).toString()).isEmpty())
						query.executeUpdate("UPDATE ligne_calcul "
								+ "SET "+columns.elementAt(j)+" = '"+ Float.parseFloat(((Vector)data.elementAt(i)).elementAt(j).toString()) 
								+"' WHERE id_ligne = '" + Integer.parseInt(((Vector)data.elementAt(i)).elementAt(0).toString()) 
								+ "' AND id_calcul = '"+id_calcul+"'");
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Retourne toutes les chambres de la base de donnes
	 * @return un tableau de donnes
	 */
	public String[] get_chambres()
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();
		
		try 
		{
			rs = query.executeQuery("SELECT nom_chambre "
					+ "FROM chambres "
					+ "ORDER BY 1;");
			while(rs.next())
				tmp.add(rs.getString(1));
			
			returned = new String[tmp.size()];
			for( int i = 0; i < tmp.size(); i++ )
				returned[i] = tmp.get(i);
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return returned;
	}
	
	/**
	 * Retourne tous les sites dont le nom commence par start
	 * @param start dbut du nom
	 * @return tableau des noms des sites
	 */
	public String[] get_sites(String start)
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();
		
		try
		{
			rs = query.executeQuery("SELECT nom_lieu "
					+ "FROM lieux "
					+ "WHERE nom_lieu LIKE '" +
						start.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "%'");
			while(rs.next())
				tmp.add(rs.getString(1));
			
			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}
			else
			{
				returned = new String[1];
				returned[0] = "";
			}
			
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Ajoute un site dans la base
	 * @param site nom du site
	 * @return true = ajout, false = non ajout
	 */
	public boolean add_site(String site)
	{
		boolean inserted = false;
		try
		{
			rs = query.executeQuery("SELECT count(*) FROM lieux WHERE nom_lieu = '" 
					+ site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "'");
			rs.next();
			if( rs.getInt(1) == 0)
			{
				query.executeUpdate("INSERT INTO lieux VALUES (NULL, '" + site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
						+ "')");
				inserted = true;
			}
		}
		catch(SQLException e){ e.printStackTrace(); };
		
		return inserted;
	}
	
	/**
	 * Retourne le nom d'auteur d'un taxon par son code
	 * @param code code du taxon
	 * @return le nom de l'auteur
	 */
	public String get_auteur(String code)
	{
		String returned = null;
		try
		{
			rss = connection.createStatement().executeQuery("SELECT nom_auteur FROM auteurs NATURAL JOIN liste_taxa WHERE code_taxon = '" + code + "'" +
					" UNION SELECT nom_auteur FROM liste_taxa_cpt WHERE code_taxon = '" + code + "'");
			
			if( rss.next() )
				returned = rss.getString(1);
			if( returned == null )
				returned = "N/A";
			
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Retourne le biovol cel. d'un taxon
	 * @param code code du taxon
	 * @return biovolcel
	 */
	public String get_biovolcel(String code, boolean perso)
	{
		String returned = null;
		try
		{
			String req = "SELECT biovolcel ";
			
			if (perso)
				req += "FROM liste_taxa_cpt ";
			else
				req += "FROM liste_taxa ";
			
			req += "WHERE code_taxon = '" + code + "'";
			
			rss = connection.createStatement().executeQuery(req);
			
			if( rss.next() )
				returned = rss.getString(1);
			if( returned == null )
				returned = "N/A";
			
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Retourne le biovol ind. d'un taxon
	 * @param code code du taxon
	 * @param perso indique si on travaille sur un taxon personnel
	 * @return biovolind
	 */
	public String get_biovolind(String code, boolean perso)
	{
		String returned = null;
		
		try
		{
			String req = "SELECT biovolind ";
			
			if (perso)
				req += "FROM liste_taxa_cpt ";
			else
				req += "From liste_taxa ";
			
			req += "WHERE code_taxon = '" + code + "'";
			
			rss = connection.createStatement().executeQuery(req);
			
			if( rss.next() )
				returned = rss.getString(1);
			if( returned == null )
				returned = "N/A";
			
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Retourne le nom de la forme du taxon
	 * @param code code du taxon
	 * @return nom forme
	 */
	public String get_forme(String code, int id_forme, boolean isCol)
	{
		String returned = null;
		if(id_forme == -1){
			try
			{
				if(isCol)
					rss = connection.createStatement().executeQuery("SELECT nom_forme FROM formes " +
							"WHERE id_forme = (SELECT id_forme_ind FROM liste_taxa WHERE code_taxon = '" + code + "')");
				else
					rss = connection.createStatement().executeQuery("SELECT nom_forme FROM formes " +
							"WHERE id_forme = (SELECT id_forme_cel FROM liste_taxa WHERE code_taxon = '" + code + "')");
				
				if( rss.next() )
					returned = rss.getString(1);
				if( returned == null )
					returned = "";
			
			}
			catch(SQLException e) { e.printStackTrace(); }
		}
		else{
			try
			{
				rss = connection.createStatement().executeQuery("SELECT nom_forme FROM formes " +
							"WHERE id_forme = '" + id_forme + "'");
				
				if( rss.next() )
					returned = rss.getString(1);
				if( returned == null )
					returned = "";
			
			}
			catch(SQLException e) { e.printStackTrace(); }
		}
		return returned;
	}
	
	
	/**
	 * Retourne formule de la forme du taxon
	 * @param code code du taxon
	 * @return formule forme
	 */
	public String get_formule(String code, int id_forme, boolean isCol)
	{
		String returned = null;
		if(id_forme == -1){
			try
			{
				if(isCol)
					rss = connection.createStatement().executeQuery("SELECT formule_forme FROM formes " +
							"WHERE id_forme = (SELECT id_forme_ind FROM liste_taxa WHERE code_taxon = '" + code + "')");
				else
					rss = connection.createStatement().executeQuery("SELECT formule_forme FROM formes " +
							"WHERE id_forme = (SELECT id_forme_cel FROM liste_taxa WHERE code_taxon = '" + code + "')");
				
				if( rss.next() )
					returned = rss.getString(1);
				if( returned == null )
					returned = "";
			
			}
			catch(SQLException e) { e.printStackTrace(); }
		}
		else{
			try
			{
				rss = connection.createStatement().executeQuery("SELECT formule_forme FROM formes " +
							"WHERE id_forme = '" + id_forme + "'");
				
				if( rss.next() )
					returned = rss.getString(1);
				if( returned == null )
					returned = "";
			
			}
			catch(SQLException e) { e.printStackTrace(); }
		}
		return returned;
	}
	
	/**
	 * Retourne l'id de la forme d'un taxon
	 * @param code code du taxon
	 * @return id forme
	 */
	public int get_idForme(String code, boolean isCol)
	{
		int returned = 0;
		try
		{
			if(isCol)
				rss = connection.createStatement().executeQuery("SELECT id_forme_ind "
						+ "FROM liste_taxa " +
						"WHERE code_taxon = '" + code + "'");
			else
				rss = connection.createStatement().executeQuery("SELECT id_forme_cel "
						+ "FROM liste_taxa " +
						"WHERE code_taxon = '" + code + "'");
			
			if( rss.next() ){
				if(rss.getString(1) != null)
					returned = Integer.parseInt(rss.getString(1));
				else
					returned = 0;
			}
			
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Retourne le nom d'un taxon par son code
	 * @param code code du taxon
	 * @return nom du taxon
	 */
	public String get_nom_taxon(String code)
	{
		String returned = null;
		try
		{
			rss = connection.createStatement().executeQuery("SELECT nom_taxon "
					+ "FROM tous_codes_taxon "
					+ "WHERE code_taxon = '" + code + "'");
			
			if( rss.next() )
				returned = rss.getString(1);
			
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Retourne le code d'un taxon  partir son nom
	 * @param nom nom du taxon 
	 * @return code du taxon
	 */
	public String get_code_taxon_nom(String nom)
	{
		String returned = null;
		try
		{
			rs = query.executeQuery("SELECT code_taxon "
					+ "FROM tous_codes_taxon "
					+ "WHERE nom_taxon = '" + nom + "'");
			
			if(  rs.next())
				returned = rs.getString(1);
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Retourne tous les noms de taxons commenant par dbut
	 * @param debut dbut du nom
	 * @return tableau des noms
	 */
	public String[] get_noms_taxon_like(String debut)
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();
		try
		{
			rs = query.executeQuery("SELECT nom_taxon "
					+ "FROM tous_codes_taxon "
					+ "WHERE nom_taxon LIKE '" 
					+ debut.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "%' "
					+ "ORDER BY nom_taxon");
			
			while(rs.next())
				tmp.add(rs.getString(1));
			
			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}	
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Retourne les codes des taxons dont le code commence par debut
	 * @param debut dbut du code
	 * @return tableau des codes 
	 */
	public String[] get_codes_taxon(String debut)
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();
		
		try
		{
			rs = query.executeQuery("SELECT code_taxon "
					+ "FROM tous_codes_taxon "
					+ "WHERE code_taxon LIKE '" 
					+ debut.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "%' "
					+ "ORDER BY code_taxon");
			
			while(rs.next())
				tmp.add(rs.getString(1));
			
			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Ajoute une station dans la base de donnes
	 * @param site nom du site
	 * @param station nom de la station
	 * @param milieu milieu de la station
	 * @return true = insere, false = non insere
	 */
	public boolean add_station(String site, String station, String milieu)
	{
		boolean inserted = false;
		try
		{
			rs = query.executeQuery("SELECT count(*) FROM stations WHERE nom_station = '" 
					+ station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "'" +
					" AND id_lieu = (SELECT id_lieu FROM lieux WHERE nom_lieu = '" 
					+ site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "')");
			rs.next();
			if( rs.getInt(1) == 0)
			{
				query.executeUpdate("INSERT INTO stations VALUES (NULL, '" 
						+ station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "', " +
						"(SELECT id_lieu FROM lieux WHERE nom_lieu = '" 
						+ site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "'), '" 
						+ milieu.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "')");
				inserted = true;
			}
		}
		catch(SQLException e){ e.printStackTrace(); };
		
		return inserted;
	}
	
	/**
	 * Retourne le milieu d'une station
	 * @param site nom du site
	 * @param station nom de la station
	 * @return le milieu
	 */
	public String get_milieu(String site, String station)
	{
		String returned = null;
		try
		{
			rs = query.executeQuery("SELECT milieu FROM stations WHERE id_lieu = " +
					"(SELECT id_lieu FROM lieux WHERE nom_lieu = '" + site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+ "') AND nom_station = '" + station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "'");
			
			if( rs.next() )
				returned = rs.getString(1);	
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Retourne toutes les stations d'un site
	 * @param site nom du site
	 * @return tableau des noms de stations
	 */
	public String[] get_stations(String site)
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();
		
		try
		{
			rs = query.executeQuery("SELECT nom_station FROM stations WHERE id_lieu = " +
					"(SELECT id_lieu FROM lieux WHERE nom_lieu = '" + site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") +
					"') ORDER BY nom_station");
			while(rs.next())
				tmp.add(rs.getString(1));
			
			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}
			else
			{
				returned = new String[1];
				returned[0] = "";
			}
			
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	/**
	 * Ajoute un taxon utilisateur
	 * @param code_taxon code du taxon 
	 * @param nom_taxon nom du taxon 
	 * @param nom_auteur nom de l'auteur
	 * @param indiv individu par dfaut
	 * @param biovolcel biovolume cellulaire
	 * @param biovolind biovolume par individu
	 * @param createur nom du crateur
	 */
	public void add_taxon_cpt(String code_taxon, String nom_taxon, String nom_auteur, String indiv, Float biovolcel,
			Float biovolind, String createur )
	{
		try
		{
			query.executeUpdate("INSERT INTO liste_taxa_cpt VALUES ( NULL, '" + code_taxon + "', '" 
					+ nom_taxon.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "', '" +
					nom_auteur.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "', '" + indiv + "', " + 
					((biovolcel != 0) ? biovolcel : "NULL") + ", " + ((biovolind != 0) ? biovolind : "NULL") + ", " + 
					((createur.isEmpty()) ? "NULL" : "'" + createur.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "'") + ")");
		}
		catch(SQLException e) { e.printStackTrace(); }
	}
	
	/**
	 * Retourne les auteurs dont le nom commence par debut
	 * @param debut debut du nom d'auteur
	 * @return tableau d'auteurs
	 */
	public String[] get_auteurs_like( String debut )
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();
		
		try
		{
			rs = query.executeQuery("SELECT nom_auteur FROM auteurs WHERE nom_auteur LIKE '" + debut + "%' " +
					"UNION SELECT nom_auteur FROM liste_taxa_cpt WHERE nom_auteur LIKE '" + debut + "%' ORDER BY nom_auteur");
			while(rs.next())
				tmp.add(rs.getString(1));
			
			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}
		}
		catch(SQLException e) { e.printStackTrace(); }
		
		return returned;
	}
	
	//met  jour commentaire COMPTAGE et non calcul
	public void set_commentaire(int idCpt, String com) 
	{
		try 
		{
			query.executeUpdate("UPDATE comptage SET commentaire = '" + com.replaceAll("'", "\\\\'").replaceAll("\\\\", "\\\\\\\\") + "' WHERE id_cpt = " + idCpt);
		} catch (SQLException e) { e.printStackTrace(); }
	}
	
	//remplace la date "    - -  " par ""  
	public void set_dateNull(int id_calcul) 
	{
		try 
		{
			query.executeUpdate("UPDATE calcul_biovol SET date = '' WHERE date='    -  -  ' AND id_calcul = " + id_calcul);
		} catch (SQLException e) { e.printStackTrace(); }
	}
	
	public void remove_row( int id_calcul, int id_line )
	{
		try
		{
			query.executeUpdate("DELETE FROM ligne_calcul WHERE id_calcul = '" + id_calcul + "' AND id_ligne = '" + id_line + "'");
		}
		catch(SQLException e){ e.printStackTrace(); };
	}
	
//supprimer toutes les lignes d'un calcul
	public void vide_rows( int id_calcul )
	{
		try
		{
			query.executeUpdate("UPDATE ligne_calcul SET d=null, d1=null, d2=null, d3=null, d4=null, d5=null,"+
					" l=null, l1=null, l2=null, l3=null, l4=null,"+
					" h=null, h1=null, h2=null, h3=null, h4=null,"+
					" a=null, b=null, w=null, nb_cellule=null, biovol_objet=null, nb_epine=null"+
					" WHERE id_calcul =" + id_calcul);
		}
		catch(SQLException e){ e.printStackTrace(); };
	}
	
	/**
	 * Ferme les connections
	 */
	public void close()
	{
		try 
		{
			query.close();
			connection.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
}