package Comptage;

import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.ArrayList;

import utils.Define;

public class Comptage_Query {

	private Connection connection = null;
	private Statement query = null;
	private ResultSet rs = null;
	private ResultSet rss = null;

	@SuppressWarnings("unused")
	private String db_link;

	public Comptage_Query(String db_link)
	{
		this.db_link = db_link;

		try
		{
			Class.forName("com.mysql.jdbc.Driver");
			String url = "jdbc:mysql://" + db_link + "/" + new Define().get_db_name();
			connection = DriverManager.getConnection(url, new Define().get_user(), new Define().get_pass());
			query = connection.createStatement();
		}
		catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}


	/**
	 * 
	 * @param code d'un taxon
	 * @return boolen qui indique si le taxon a dj t cr personellement
	 */
	public boolean already_created(String code)
	{
		boolean ac = true;

		try 
		{
			rs = query.executeQuery("SELECT count(*) "
					+ "FROM liste_taxa_cpt "
					+ "WHERE code_taxon = '" + code + "'");
			rs.next();
			if( rs.getInt(1) == 0 )
				ac = false;
		} 
		catch (SQLException e) {
			e.printStackTrace();
		}

		return ac;
	}


	/**
	 * 
	 * @param id_cpt
	 * @param taxi
	 * @return la liste d'une certaine taxinomie de tous les taxons d'un comptage
	 */
	public String[] refresh_taxinomie( int id_cpt, String taxi )
	{
		String[] returned = null;
		ArrayList<String> data = new ArrayList<String>();
		String taxinomie = null;

		if( taxi.compareTo("Rgne") == 0 ) taxinomie = "regne";
		if( taxi.compareTo("Sous rgne") == 0 ) taxinomie = "sous_regne";
		if( taxi.compareTo("Embranchement") == 0 ) taxinomie = "embranchement";
		if( taxi.compareTo("Classe") == 0 ) taxinomie = "classe";
		if( taxi.compareTo("Ordre") == 0 ) taxinomie = "ordre";
		if( taxi.compareTo("Sous ordre") == 0 ) taxinomie = "sous_ordre";
		if( taxi.compareTo("Famille") == 0 ) taxinomie = "famille";

		try 
		{
			String quer;


			// si le niveau taxinomique choisi est embranchement, alors, la requte n'est pas la mme :
			// il faut prendre en compte les taxons personnels qui possdent maintenant un embranchement.

			if(taxinomie.equals("embranchement")){

				quer = 	"SELECT " + taxinomie
						+ 	" FROM ( "
						+ 		" (SELECT taxinomie.embranchement, ligne_cpt.code_taxon"
						+ 		" FROM ligne_cpt"
						+ 		" NATURAL JOIN liste_taxa"
						+ 		" NATURAL JOIN taxinomie"
						+ 		" WHERE id_cpt = " + id_cpt + " )"
						+ 		" UNION"
						+ 		" (SELECT liste_taxa_cpt.embranchement, ligne_cpt.code_taxon"
						+ 		" FROM ligne_cpt"
						+ 		" NATURAL JOIN liste_taxa_cpt"
						+ 		" WHERE id_cpt = " + id_cpt + " )"
						+ 	") AS ensemble"
						+ 	" GROUP BY ensemble.code_taxon"
						+ 	" ORDER BY ensemble.code_taxon";

			}
			else quer = "SELECT " + taxinomie + " FROM (ligne_cpt "
					+ "LEFT OUTER JOIN liste_taxa ON ligne_cpt.code_taxon = liste_taxa.code_taxon) "
					+ "LEFT OUTER JOIN taxinomie ON liste_taxa.id_taxinomie = taxinomie.id_taxinomie "
					+ "WHERE id_cpt = " + id_cpt
					+ " ORDER BY ligne_cpt.code_taxon";

			rs = query.executeQuery(quer);

			while( rs.next() )
			{
				String tax = rs.getString(1);
				if( tax == null )
					tax = "";
				data.add(tax);
			}

			returned = new String[data.size()];

			for( int i = 0; i < data.size(); i++)
				returned[i] = data.get(i);
		} 
		catch (SQLException e) {
			e.printStackTrace();
		}

		return returned;
	}

	/**
	 * 
	 * @param code
	 * @return le code sandre d'un taxon donn
	 */
	public String get_sandre( String code )
	{
		String returned = "";
		try
		{
			rss = connection.createStatement().executeQuery("SELECT code_sandre "
					+ "FROM liste_taxa "
					+ "WHERE code_taxon = '" + code + "'");

			if( rss.next() )
				returned = rss.getString(1);
			if( returned == null )
				returned = "";

		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}	


	/**
	 * 
	 * @param id_cpt
	 * @param taxi
	 * @return toutes les lignes de comptage d'un comptage donn
	 */
	public Object[][] get_all_for_consult( int id_cpt, String taxi )
	{
		Object[][] returned = null;
		ArrayList<Object> data = new ArrayList<Object>();
		String code_taxon;
		float multi = this.get_multi(id_cpt);
		int index = 0;

		String[] d = this.refresh_taxinomie(id_cpt, taxi);


		// formats dcimaux pour les abondances et le biovolume
		DecimalFormat dfAb = (DecimalFormat) DecimalFormat
				.getNumberInstance();
		dfAb.applyPattern("0.00");

		DecimalFormat dfBv = (DecimalFormat) DecimalFormat
				.getNumberInstance();
		dfBv.applyPattern("0.00000");

		DecimalFormatSymbols symbols = new DecimalFormatSymbols();
		symbols.setDecimalSeparator('.');
		dfAb.setDecimalFormatSymbols(symbols);
		dfBv.setDecimalFormatSymbols(symbols);


		try 
		{
			String quer = "SELECT code_taxon, type, cf, nb_cpt, biovol_calcule "
					+ "FROM ligne_cpt "
					+ "WHERE id_cpt = " + id_cpt 
					+ " order by code_taxon";

			rs = query.executeQuery(quer);

			while( rs.next() )
			{
				code_taxon = rs.getString("code_taxon");

				String sandre = this.get_sandre(code_taxon);
				if( sandre == "" )
					data.add(Integer.MAX_VALUE);
				else
					data.add(Integer.parseInt(sandre));
				data.add(code_taxon);
				data.add(rs.getString("type"));
				data.add( ((rs.getBoolean("cf")) ? "Cf." : "") );
				data.add(this.get_nom_taxon(code_taxon));
				data.add(this.get_auteur(code_taxon));
				data.add(this.get_actuel(code_taxon));
				data.add(d[index]);

				// on va regarder si la valeur nb moy cell/ind est renseigne
				Integer n = null;
				n = get_nb_moy_cell_ind(id_cpt, code_taxon);

				// et calculer les abondances en fonction
				Float abondance1; // nb cell/ml
				Float abondance2; // nb ind/ml

				// rcupration du type d'objet compt
				String typeObjCpt = get_type_cpt(id_cpt, code_taxon);

				// si la valeur du nb moy cell/ind est renseigne
				if(n != null){
					// si le taxon est compt en cellules
					if(typeObjCpt.equals("Cel.")){
						// alors les deux abondances peuvent tre calcules
						abondance1 = (rs.getInt("nb_cpt") * multi * 10 * 1000000)/10f;						
						abondance2 = ((rs.getInt("nb_cpt") * multi * 10 * 1000000)/10f) / n;						
					}
					// si le taxon n'a pas t compt en cellules
					else {						
						abondance1 = ((rs.getInt("nb_cpt") * multi * 10 * 1000000)/10f)*n;						
						abondance2 = (rs.getInt("nb_cpt") * multi * 10 * 1000000)/10f;
					}					
				}
				// si on ne connait pas le nb moy cell/ind :
				else {
					// si le taxon a t compt en cellules
					if(typeObjCpt.equals("Cel.")){
						// on ne peut connaitre qu'une abondance :  nb cell/ml						
						abondance1 = (rs.getInt("nb_cpt") * multi * 10 * 1000000)/10f;						
						abondance2 = -1f; // -1 signifie que cette valeur n'a pas t calcule
					}
					// si le taxon n'a pas t compt en cellules
					else {
						// on ne peut connaitre qu'une abondance : nb ind/ml
						abondance1 = -1f; // -1 signifie que cette valeur n'a pas t calcule
						abondance2 = (rs.getInt("nb_cpt") * multi * 10 * 1000000)/10f;
					}
				}

				data.add(abondance1);
				data.add(abondance2);

				Float bio = rs.getFloat("biovol_calcule");

				if( bio == 0.0 )
					data.add(0.0f);
				else{									
					Float bvCalc = (bio * 10000) / 10000f;
					data.add(bvCalc);					
				}

				index++;
			}

			returned = new Object[data.size()/11][11];

			for( int i = 0; i < data.size(); i++)
				returned[i/11][i%11] = data.get(i);
		} 
		catch (SQLException e) {
			e.printStackTrace();
		}

		return returned;
	}


	/**
	 * Retourne tous les taxons d'un comptage
	 * @param id_cpt id du comptage
	 * @return une matrice de donnes
	 */
	public Object[][] get_all_taxon( int id_cpt )
	{
		Object[][] returned = null;
		ArrayList<Object> data = new ArrayList<Object>();
		String code;

		// les biovolumes null vaudront 0.0
		float biovol = 0f;

		boolean user = false;
		try 
		{
			ResultSet rs = query.executeQuery("SELECT id_ligne, code_taxon, "
					+ "nb_cell_moy_ind, nb_cpt, type, cf, taxon_cpt " +
					" FROM ligne_cpt "
					+ "WHERE id_cpt = " + id_cpt);

			while( rs.next() )
			{
				code = rs.getString("code_taxon");
				user = rs.getBoolean("taxon_cpt");

				data.add(rs.getInt("id_ligne"));
				data.add(code);
				data.add(rs.getString("type"));
				data.add( ((rs.getBoolean("cf")) ? "Cf." : "") );

				data.add(this.get_nom_taxon(code));

				data.add(this.get_auteur(code));
				data.add(this.get_actuel(code));

				if(this.get_biovol(code, rs.getString("type"), user) != null)
					biovol = Float.parseFloat(this.get_biovol(code, rs.getString("type"), user));

				data.add(biovol);

				// si le nombre moy de cell/ind est dfini on le met, sinon, on le met  null
				if(rs.getInt("nb_cell_moy_ind") > 0)
					data.add(rs.getInt("nb_cell_moy_ind"));/*#*/
				else
					data.add(null);/*#*/

				data.add(rs.getInt("nb_cpt"));
				data.add(user);
			}

			returned = new Object[data.size()/11][11];/*#*/

			for( int i = 0; i < data.size(); i++)
				returned[i/11][i%11] = data.get(i);/*#*/
		} 
		catch (SQLException e) {
			e.printStackTrace();
		}

		return returned;
	}

	/**
	 * Retourne le facteur multiplicatif du calcul du biovolume
	 * @param id_cpt id du comptage
	 * @return un flottant
	 */
	public float get_multi(int id_cpt)
	{
		float sur_ch = 0, dill = 0, sur_cp = 0, vol_sed = 0, multipliant = 0;
		int nb_c = 0;

		try
		{
			rss = connection.createStatement().executeQuery(
					"SELECT surface_chambre, facteur_dillution, surface_champ, "
							+ "vol_sedimente, nb_champs " 
							+ "FROM chambres "
							+ "NATURAL JOIN comptage "
							+ "WHERE id_cpt = " + id_cpt);

			rss.next();

			sur_ch = rss.getFloat("surface_chambre");
			dill = rss.getFloat("facteur_dillution");
			sur_cp = rss.getFloat("surface_champ");
			vol_sed = rss.getFloat("vol_sedimente");
			nb_c = rss.getInt("nb_champs");

			multipliant = (sur_ch * dill) / (sur_cp * vol_sed * nb_c * 1000000);
		}
		catch(SQLException e){ e.printStackTrace(); }

		return multipliant;
	}

	/**
	 * Calcule le biovolume en mm/l
	 * @param id_cpt id du comptage
	 * @param id_l id de la ligne 
	 * @param biovol biovolume par individu
	 * @param multipliant facteur multiplicatif
	 * @param perso indique si on travaille sur un taxon personnel
	 */
	public void set_biovol(int id_cpt, int id_l, float biovol, float multipliant, boolean perso)
	{
		int nb_cpt = 0;
		String type_obj_cpt = null;
		int nb_moy_cell_ind;
		Float biovolcel;
		Float biovolind;

		try
		{			
			String requete ="SELECT nb_cpt, type, nb_cell_moy_ind, biovolcel, biovolind "
					+ "FROM ligne_cpt ";
			if(!perso)
				requete += "NATURAL JOIN liste_taxa ";
			else
				requete += "NATURAL JOIN liste_taxa_cpt ";

			requete += "WHERE id_ligne = " + id_l + " AND id_cpt = " + id_cpt;


			rs = query.executeQuery(requete);
			rs.next();
			// rcupration des valeurs
			nb_cpt = rs.getInt("nb_cpt");
			type_obj_cpt = rs.getString("type");
			nb_moy_cell_ind = rs.getInt("nb_cell_moy_ind");
			biovolcel = rs.getFloat("biovolcel");
			biovolind = rs.getFloat("biovolind");


			// nouvelle modification importante !
			// si le biovolume du type compt n'est pas renseign 
			//		ou si on connait le biovolume cellulaire du taxon
			// 		car dans tous les cas, c'est mieux de calculer avec le biovol cel :
			if(biovol == 0 || biovolcel != 0 ){
				// on va voir premirement si on connait le nb moy cell/ind
				// si cette valeur est connue on continue	
				if(nb_moy_cell_ind != 0){
					// si l'individu a t compt en cellules
					if( type_obj_cpt.equals("Cel.") ){
						//on va voir si il a un biovolind connu
						if (biovolind != 0 && biovolcel == 0){
							biovol = biovolind / nb_moy_cell_ind;
						}
					}
					// si l'individu n'a pas t compt en cellules
					else{
						// on va voir si il a un biovolcell connu
						if (biovolcel != 0){
							biovol = biovolcel * nb_moy_cell_ind;
						}
					}
				}
			}

			Float biovol_calc = biovol * multipliant * new Float(nb_cpt);

			// mise  jour de la table ligne_cpt avec le nouveau biovolume calcul !
			query.executeUpdate("UPDATE ligne_cpt "
					+ "SET biovol_calcule = " + biovol_calc + 
					" WHERE id_cpt = " + id_cpt + " AND id_ligne = " + id_l);
		}
		catch(SQLException e){ e.printStackTrace(); }
	}

	/**
	 * Retourne les donnes du cartouche d'un comptage
	 * @param id_cpt id du comptage
	 * @return un tableau de donnes
	 */
	public String[] get_all_up_for_open( int id_cpt )
	{
		String[] returned = new String[12];
		String com;

		try
		{
			rs = query.executeQuery("SELECT nom_lieu, nom_station, milieu, date_prelevement, "
					+ "(SELECT ref_etude FROM echantillons WHERE echantillons.id_echantillon = "
					+ "(SELECT id_echantillon FROM comptage WHERE id_cpt = '" + id_cpt + "')) AS ref_etude, "
					+ "nom_operateur, vol_sedimente, facteur_dillution, nom_chambre, "
					+ "nb_champs, nb_indiv_total, commentaire " 
					+ "FROM lieux "
					+ "NATURAL JOIN stations "
					+ "NATURAL JOIN echantillons "
					+ "NATURAL JOIN comptage "
					+ "NATURAL JOIN chambres " +
					"WHERE id_cpt = " + id_cpt);

			rs.next();


			// rcupration des donnes
			returned[0] = rs.getString("nom_lieu");
			returned[1] = rs.getString("nom_station");
			returned[2] = rs.getString("milieu");
			returned[3] = rs.getString("date_prelevement");
			returned[4] = rs.getString("ref_etude");
			returned[5] = rs.getString("nom_operateur");
			returned[6] = Float.toString(rs.getFloat("vol_sedimente"));
			returned[7] = Float.toString(rs.getFloat("facteur_dillution"));
			returned[8] = rs.getString("nom_chambre");
			returned[9] = Integer.toString(rs.getInt("nb_champs"));
			returned[10] = Integer.toString(rs.getInt("nb_indiv_total"));
			com = rs.getString("commentaire");
			if( com == null ) com = "";
			returned[11] = com;
		}
		catch(SQLException e){ e.printStackTrace(); }

		return returned;
	}

	/**
	 * Change le champ cf d'un taxon d'un comptage
	 * @param id_cpt id du comptage
	 * @param id_ligne id de la ligne du taxon
	 * @param cf true = cf, false = non cf
	 */
	public void change_cf( int id_cpt, int id_ligne, boolean cf )
	{
		try
		{
			query.executeUpdate("UPDATE ligne_cpt SET cf = " + ((cf)?"1":"0") 
					+ " WHERE id_cpt = " + id_cpt + " AND id_ligne = " + id_ligne);
		}
		catch(SQLException e){ e.printStackTrace(); };
	}

	/**
	 * Change le type d'individu par dfaut d'un taxon d'un comptage et retourne le biovolume
	 * associ  ce type
	 * @param id_cpt id du comptage
	 * @param id_ligne id de la ligne du taxon
	 * @param type type d'indiv par dfaut (Cel., Fil., Col.)
	 * @param code_taxon code du taxon
	 * @return biovolume associ au type
	 */
	public float change_type( int id_cpt, int id_ligne, String type, String code_taxon, boolean user )
	{
		float returned = 0;
		try
		{
			query.executeUpdate("UPDATE ligne_cpt SET type = '" + type 
					+ "' WHERE id_cpt = " + id_cpt + " AND id_ligne = " + id_ligne);

			String t = ( (type.compareTo("Cel.") == 0) ? "biovolcel" : "biovolind");

			String req = "SELECT " + t + " FROM liste_taxa_cpt "
					+ "WHERE code_taxon = '" + code_taxon + "'";

			if( ! user )
				req = "SELECT " + t 
				+ " FROM liste_taxa WHERE code_taxon = '" + code_taxon + 
				"' UNION " + req;

			rs = query.executeQuery(req);
			rs.next();
			returned = rs.getFloat(1);
		}
		catch(SQLException e){ e.printStackTrace(); };
		return returned;
	}

	/**
	 * Change le nombre total d'individu compt
	 * @param id_cpt id du comptage
	 * @param nb nombre d'individu compt
	 */
	public void change_nb_tot( int id_cpt, int nb )
	{
		try
		{
			query.executeUpdate("UPDATE comptage SET nb_indiv_total = " + nb 
					+ " WHERE id_cpt = " + id_cpt);
		}
		catch(SQLException e){ e.printStackTrace(); };
	}

	/**
	 * Change le nombre de champs compts
	 * @param id_cpt id du comptage
	 * @param nb nombre de champs
	 */
	public void change_nb_chp( int id_cpt, int nb )
	{
		try
		{
			query.executeUpdate("UPDATE comptage "
					+ "SET nb_champs = " + nb 
					+ " WHERE id_cpt = " + id_cpt);
		}
		catch(SQLException e){ e.printStackTrace(); };
	}

	/**
	 * Change le nombre d'individu compt par ligne et calcule le biovolume
	 * @param id_cpt id du comptage
	 * @param id_ligne id de la ligne 
	 * @param nb nombre d'individu compt
	 * @param biovol biovolume du type selectionn
	 */
	public void change_nb_cpt( int id_cpt, int id_ligne, int nb )
	{
		try
		{
			query.executeUpdate("UPDATE ligne_cpt "
					+ "SET nb_cpt = " + nb 
					+ " WHERE id_cpt = " + id_cpt + " AND id_ligne = " + id_ligne);
		}
		catch(SQLException e){ e.printStackTrace(); };
	}


	/**
	 * Change le nombre moyen de cellules par individu
	 * @param id_cpt id du comptage
	 * @param id_ligne id de la ligne 
	 * @param nb nombre moyen de cellules par individu
	 */
	public void change_nb_moy_cell_ind( int id_cpt, int id_ligne, int nb ){
		try{
			if (nb > 0)
				query.executeUpdate("UPDATE ligne_cpt SET nb_cell_moy_ind = " + nb + 
						" WHERE id_cpt = " + id_cpt + " AND id_ligne = " + id_ligne);
			else 
				query.executeUpdate("UPDATE ligne_cpt SET nb_cell_moy_ind = null" + 
						" WHERE id_cpt = " + id_cpt + " AND id_ligne = " + id_ligne);
		}
		catch(SQLException e){e.printStackTrace();};
	}

	/**
	 * Supprime une ligne d'un comptage
	 * @param id_cpt id du comptage
	 * @param id_line id de la ligne
	 */
	public void remove_row( int id_cpt, int id_line )
	{
		try
		{
			query.executeUpdate("DELETE FROM ligne_cpt WHERE id_cpt = " + id_cpt + " AND id_ligne = " + id_line);
		}
		catch(SQLException e){ e.printStackTrace(); };
	}

	/**
	 * Ajoute une ligne dans un comptage
	 * @param id_cpt id du comptage
	 * @param id_line id de la ligne 
	 * @param code code du taxon 
	 * @param type type d'individu
	 * @return true = ajout, false = non ajout
	 */
	public boolean add_line(int id_cpt, int id_line, String code, String type )
	{
		boolean inserted = false;
		try
		{
			query.executeUpdate("INSERT INTO ligne_cpt VALUES (" + id_line + ", " + id_cpt + ", '" 
					+ code + "', 0, '" + type + "', 0, NULL, NULL, 0)");
			inserted = true;
		}
		catch(SQLException e){ e.printStackTrace(); };

		return inserted;
	}

	/**
	 * Retourne toutes les chambres de la base de donnes
	 * @return un tableau de donnes
	 */
	public String[] get_chambres()
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();

		try 
		{
			rs = query.executeQuery("SELECT nom_chambre FROM chambres ORDER BY 1;");
			while(rs.next())
				tmp.add(rs.getString(1));

			returned = new String[tmp.size()];
			for( int i = 0; i < tmp.size(); i++ )
				returned[i] = tmp.get(i);

		} catch (SQLException e) {
			e.printStackTrace();
		}

		return returned;
	}

	/**
	 * Retourne tous les sites dont le nom commence par start
	 * @param start dbut du nom
	 * @return tableau des noms des sites
	 */
	public String[] get_sites(String start)
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();

		try
		{
			rs = query.executeQuery("SELECT nom_lieu FROM lieux WHERE nom_lieu LIKE '" +
					start.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "%'");
			while(rs.next())
				tmp.add(rs.getString(1));

			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}
			else
			{
				returned = new String[1];
				returned[0] = "";
			}

		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}

	/**
	 * Ajoute un site dans la base
	 * @param site nom du site
	 * @return true = ajout, false = non ajout
	 */
	public boolean add_site(String site)
	{
		boolean inserted = false;
		try
		{	
			//vrifie le cas "champ vide"
			if (!site.isEmpty()){		
				rs = query.executeQuery("SELECT count(*) "
						+ "FROM lieux WHERE nom_lieu = '" 
						+ site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "'");
				rs.next();
				if( rs.getInt(1) == 0)
				{
					query.executeUpdate("INSERT INTO lieux "
							+ "VALUES (NULL, '" + site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
							+ "')");
					inserted = true;
				}
			}
		}
		catch(SQLException e){ e.printStackTrace(); };

		return inserted;
	}

	/**
	 * Retourne le code taxon de ref d'un taxon
	 * @param code code du taxon 
	 * @return code du taxon de reference
	 */
	public String get_actuel(String code)
	{
		String returned = null;
		try
		{
			rss = connection.createStatement().executeQuery("SELECT code_taxon_ref "
					+ "FROM synonymes "
					+ "WHERE code_taxon = '" + code + "'");

			if( rss.next() )
			{
				returned = rss.getString(1);
				if( returned.compareTo(code) == 0 )
					returned = "";
			}
			else
				returned = "";
		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}

	/**
	 * Retourne le nom d'auteur d'un taxon par son code
	 * @param code code du taxon
	 * @return le nom de l'auteur
	 */
	public String get_auteur(String code)
	{
		String returned = null;
		try
		{
			rss = connection.createStatement().executeQuery("SELECT nom_auteur "
					+ "FROM auteurs "
					+ "NATURAL JOIN liste_taxa "
					+ "WHERE code_taxon = '" + code + "'" +
					" UNION "
					+ "SELECT nom_auteur "
					+ "FROM liste_taxa_cpt "
					+ "WHERE code_taxon = '" + code + "'");

			if( rss.next() )
				returned = rss.getString(1);
			if( returned == null )
				returned = "";

		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}

	/**
	 * Retourne le nom d'un taxon par son code
	 * @param code code du taxon
	 * @return nom du taxon
	 */
	public String get_nom_taxon(String code)
	{
		String returned = null;
		try
		{
			rss = connection.createStatement().executeQuery("SELECT nom_taxon "
					+ "FROM tous_codes_taxon "
					+ "WHERE code_taxon = '" + code + "'");

			if( rss.next() )
				returned = rss.getString(1);

		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}

	/**
	 * Retourne le code d'un taxon par son nom
	 * @param nom nom du taxon 
	 * @return code du taxon
	 */
	public String get_code_taxon_nom(String nom)
	{
		String returned = null;
		try
		{
			rs = query.executeQuery("SELECT code_taxon "
					+ "FROM tous_codes_taxon "
					+ "WHERE nom_taxon = '" + nom + "'");

			if(  rs.next())
				returned = rs.getString(1);
		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}

	/**
	 * Retourne tous les noms de taxons commenant par dbut
	 * @param debut dbut du nom
	 * @return tableau des noms
	 */
	public String[] get_noms_taxon_like(String debut)
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();
		try
		{
			rs = query.executeQuery("SELECT nom_taxon "
					+ "FROM tous_codes_taxon "
					+ "WHERE nom_taxon LIKE '" 
					+ debut.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "%' "
					+ "ORDER BY nom_taxon");

			while(rs.next())
				tmp.add(rs.getString(1));

			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}	
		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}

	/**
	 * Retourne le biovol du type par dfaut d'un taxon
	 * @param code code du taxon
	 * @return biovolcel ou biovolind
	 */
	public String get_biovol(String code, String type, boolean user)
	{
		String returned = null;
		try
		{			
			String ind_def = "";
			if( type.compareTo("") == 0 )
				ind_def = this.get_indiv_def(code);
			else
				ind_def = type;
			String t = ((ind_def.compareTo("Cel.") == 0) ? "biovolcel" : "biovolind");
			String req = "SELECT " + t + " FROM liste_taxa_cpt WHERE code_taxon = '" + code + "'";
			if( ! user )
				req = "SELECT " + t + " FROM liste_taxa WHERE code_taxon = '" + code + 
				"' UNION " + req;

			rss = connection.createStatement().executeQuery(req);

			if(  rss.next() )
				returned = Float.toString(rss.getFloat(1));

		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}

	/**
	 * Retourne le type d'individu par dfaut d'un taxon
	 * @param code code du taxon
	 * @return le type d'indiv par dfaut (Cel., Fil., Col.)
	 */
	public String get_indiv_def(String code)
	{
		String returned = null;
		try
		{
			rss = connection.createStatement().executeQuery("SELECT indiv_defaut "
					+ "FROM liste_taxa "
					+ "WHERE code_taxon = '" + code + "'" +
					" UNION "
					+ "SELECT indiv_defaut "
					+ "FROM liste_taxa_cpt "
					+ "WHERE code_taxon = '" + code + "'");

			if(  rss.next())
				returned = rss.getString(1);
		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}

	/**
	 * Retourne les codes des taxons dont le code commence par debut
	 * @param debut dbut du code
	 * @return tableau des codes 
	 */
	public String[] get_codes_taxon(String debut)
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();

		try
		{
			rs = query.executeQuery("SELECT code_taxon "
					+ "FROM tous_codes_taxon "
					+ "WHERE code_taxon LIKE '" 
					+ debut.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "%' "
					+ "ORDER BY code_taxon");
			while(rs.next())
				tmp.add(rs.getString(1));

			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}
		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}

	/**
	 * Ajoute une station dans la base de donnes
	 * @param site nom du site
	 * @param station nom de la station
	 * @param milieu milieu de la station
	 * @return true = insere, false = non insere
	 */
	public boolean add_station(String site, String station, String milieu)
	{
		boolean inserted = false;
		try
		{
			//vrifie le cas "champ vide"
			if (!station.isEmpty()){		
				rs = query.executeQuery("SELECT count(*) FROM stations WHERE nom_station = '" 
						+ station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "'" +
						" AND id_lieu = (SELECT id_lieu FROM lieux WHERE nom_lieu = '" 
						+ site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "')");
				rs.next();
				if( rs.getInt(1) == 0)
				{
					query.executeUpdate("INSERT INTO stations VALUES (NULL, '" 
							+ station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "', " +
							"(SELECT id_lieu FROM lieux WHERE nom_lieu = '" 
							+ site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "'), '" 
							+ milieu.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "')");
					inserted = true;
				}
			}
		}
		catch(SQLException e){ e.printStackTrace(); };

		return inserted;
	}

	/**
	 * Retourne le milieu d'une station
	 * @param site nom du site
	 * @param station nom de la station
	 * @return le milieu
	 */
	public String get_milieu(String site, String station)
	{
		String returned = null;
		try
		{
			rs = query.executeQuery("SELECT milieu FROM stations WHERE id_lieu = " +
					"(SELECT id_lieu FROM lieux WHERE nom_lieu = '" + site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+ "') AND nom_station = '" + station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "'");

			if( rs.next() )
				returned = rs.getString(1);	
		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}


	/**
	 * Retourne le nombre moyens de cellules moyen par individu d'un taxon d'un comptage
	 * @param id_cpt identifiant du comptage
	 * @param code_taxon code du taxon
	 * @return nb moy cell/ind du taxon
	 */	
	public Integer get_nb_moy_cell_ind(int id_cpt, String code_taxon){
		Integer result = null;

		try
		{
			rss = connection.createStatement().executeQuery("SELECT nb_cell_moy_ind FROM ligne_cpt"
					+ " WHERE id_cpt = " + id_cpt + " AND code_taxon like '" + code_taxon + "'");

			if( rss.next() )
				result = rss.getInt(1);	
		}
		catch(SQLException e) { e.printStackTrace(); }

		if(result == 0)
			result = null;

		return result;
	}

	/**
	 * Retourne le type compt d'un taxon d'un comptage
	 * @param id_cpt identifiant du comptage
	 * @param code_taxon code du taxon
	 * @return le type compt (Cel., Fil., ou Col.)
	 */	
	public String get_type_cpt(int id_cpt, String code_taxon){
		String result = null;

		try
		{
			rss = connection.createStatement().executeQuery("SELECT type FROM ligne_cpt"
					+ " WHERE id_cpt = " + id_cpt + " AND code_taxon like '" + code_taxon + "'");

			if( rss.next() )
				result = rss.getString(1);	
		}
		catch(SQLException e) { e.printStackTrace(); }

		return result;		
	}


	/**
	 * Ajoute un comptage dans la base et cre l'chantillon associ
	 * @param site nom du site
	 * @param station nom de la station 
	 * @param date date de prlvement de l'chantillon
	 * @param ref ref de l'chantillon
	 * @param oper oprateur ayant fait la saisie
	 * @param volsed volume sdiment
	 * @param dill facteur de dillution
	 * @param ch nom de la chambre de comptage
	 * @return id du comptage cr
	 */
	public int add_cpt(String site, String station, String date, String ref, String oper, float volsed, float dill, String ch)
	{
		int id_ech, id_station, id_ch, id_cpt = 0;
		try
		{
			rs = query.executeQuery("SELECT id_station FROM stations WHERE nom_station = '" +
					station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "' AND "
					+ " id_lieu = (SELECT id_lieu FROM lieux WHERE nom_lieu = '" + site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+ "')");

			rs.next();
			id_station = rs.getInt(1);

			rs = query.executeQuery("SELECT id_chambre FROM chambres WHERE nom_chambre = '" 
					+ ch.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "'");
			rs.next();
			id_ch = rs.getInt(1);

			query.executeUpdate("INSERT INTO echantillons VALUES (NULL, '" + date + "', '" 
					+ ref.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "', " + id_station + ")",
					Statement.RETURN_GENERATED_KEYS);
			rs = query.getGeneratedKeys();
			rs.next();
			id_ech = rs.getInt(1);
			if( id_ech == 0 )
				return 0;

			query.executeUpdate("INSERT INTO comptage VALUES (NULL, " + id_ech + ", '" 
					+ new SimpleDateFormat("yyyy-MM-dd").format(new java.util.Date()) + "', " + volsed + ", " + dill + ", 1, 0, '" 
					+ oper.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "', " + id_ch + ", NULL)", Statement.RETURN_GENERATED_KEYS);
			rs = query.getGeneratedKeys();
			rs.next();
			id_cpt = rs.getInt(1);

		}
		catch( SQLException e) { e.printStackTrace(); }

		return id_cpt;
	}

	/**
	 * Modifie un comptage dans la base et l'chantillon associ
	 * @param id_cpt identifiant du comptage
	 * @param site nom du site
	 * @param station nom de la station 
	 * @param date date de prlvement de l'chantillon
	 * @param ref ref de l'chantillon
	 * @param oper oprateur ayant fait la saisie
	 * @param volsed volume sdiment
	 * @param dill facteur de dillution
	 * @param ch nom de la chambre de comptage
	 * @return id du comptage cr
	 */
	public void set_cpt(int id_cpt, String site, String station, String date, 
			String ref, String oper, float volsed, float dill, String ch){

		int id_ech, id_station, id_ch;

		try {
			// rcupration de l'identifiant de la station associe
			rs = query.executeQuery("SELECT id_station FROM stations WHERE nom_station = '" +
					station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "' AND "
					+ " id_lieu = (SELECT id_lieu FROM lieux WHERE nom_lieu = '" + site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+ "')");

			rs.next();
			id_station = rs.getInt(1);


			// rcupration de l'identifiant de l'chantillon associ
			rs = query.executeQuery("SELECT id_echantillon FROM comptage "
					+ " WHERE id_cpt = " + id_cpt);
			rs.next();
			id_ech = rs.getInt(1);



			// rcupration de l'identifiant de la chambre associe
			rs = query.executeQuery("SELECT id_chambre FROM chambres WHERE nom_chambre = '" 
					+ ch.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "'");
			rs.next();
			id_ch = rs.getInt(1);		

			// modification de l'echantillon associ au comptage
			query.executeUpdate("UPDATE echantillons "
					+ "SET date_prelevement = '" + date + "', "
					+ "ref_etude = '" + ref.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "', "
					+ "id_station = " + id_station
					+ " WHERE id_echantillon = " + id_ech);


			// modification du comptage concern
			query.executeUpdate("UPDATE comptage "
					+ "SET  id_echantillon = " + id_ech
					+ ", vol_sedimente = " + volsed
					+ " , facteur_dillution = " + dill
					+ ", nom_operateur = '" + oper.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+ "', id_chambre = " + id_ch
					+ " WHERE id_cpt = " + id_cpt);
		}
		catch( SQLException e) { e.printStackTrace(); }

	}



	public void delCpt(int id_cpt)
	{
		try
		{
			//suppression d'un comptage, et des lignes associes
			query.executeUpdate("DELETE FROM ligne_cpt WHERE id_cpt = '"+id_cpt+"'");
			query.executeUpdate("DELETE FROM comptage WHERE id_cpt = '"+id_cpt+"'");
		}
		catch(SQLException e){ e.printStackTrace(); };
	}

	/**
	 * Retourne toutes les stations d'un site
	 * @param site nom du site
	 * @return tableau des noms de stations
	 */
	public String[] get_stations(String site)
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();

		try
		{
			rs = query.executeQuery("SELECT nom_station FROM stations WHERE id_lieu = " +
					"(SELECT id_lieu FROM lieux WHERE nom_lieu = '" + site.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") +
					"') ORDER BY nom_station");
			while(rs.next())
				tmp.add(rs.getString(1));

			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}
			else
			{
				returned = new String[1];
				returned[0] = "";
			}

		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}

	/**
	 * Retourne toutes les infos de tous les comptages pour la fenetre d'ouverture d'un comptage
	 * @return une matrice de donnes
	 */
	public Object[][] get_data()
	{
		Object[][] returned = null;
		ArrayList<Object> data = new ArrayList<Object>();

		try 
		{
			rs = query.executeQuery("SELECT nom_lieu, nom_station, date_cpt, ref_etude, date_prelevement, nom_operateur, id_cpt" +
					" FROM lieux NATURAL JOIN stations NATURAL JOIN echantillons NATURAL JOIN comptage ORDER BY nom_lieu, nom_station, date_prelevement, ref_etude");

			while( rs.next() )
			{
				data.add(rs.getString("nom_lieu"));
				data.add(rs.getString("nom_station"));
				data.add(rs.getString("date_prelevement"));
				data.add(rs.getString("ref_etude"));
				data.add(rs.getString("date_cpt"));
				data.add(rs.getString("nom_operateur"));
				data.add(rs.getString("id_cpt"));
			}

			returned = new Object[data.size()/7][7];

			for( int i = 0; i < data.size(); i++)
				returned[i/7][i%7] = (Object)data.get(i);

		} 
		catch (SQLException e) {
			e.printStackTrace();
		}

		return returned;
	}

	/**
	 * Retourne la liste des taxons rencontrs sur une station d'un site donn
	 * @return une matrice de donnes
	 */
	public Object[][] get_taxon(String lieu, String station)
	{
		Object[][] returned = null;
		ArrayList<Object> data = new ArrayList<Object>();

		try 
		{
			rs = query.executeQuery("select ligne_cpt.code_taxon, nom_taxon, nom_auteur, code_taxon_ref, cf" +
					" from ligne_cpt" +
					" inner join comptage on ligne_cpt.id_cpt= comptage.id_cpt" +
					" inner join echantillons on comptage.id_echantillon=echantillons.id_echantillon" +
					" inner join stations on echantillons.id_station=stations.id_station" +
					" inner join lieux on stations.id_lieu=lieux.id_lieu" +
					" inner join liste_taxa on ligne_cpt.code_taxon=liste_taxa.code_taxon" +
					" inner join auteurs on liste_taxa.id_auteur=auteurs.id_auteur" +
					" inner join synonymes on liste_taxa.code_taxon=synonymes.code_taxon" +
					" where nom_lieu like '" + lieu.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+"'"+ 
					" and nom_station like '"+ station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+"'"+
					" union" +
					" select ligne_cpt.code_taxon, nom_taxon, nom_auteur, code_taxon_ref, cf" +
					" from ligne_cpt" +
					" inner join comptage on ligne_cpt.id_cpt= comptage.id_cpt" +
					" inner join echantillons on comptage.id_echantillon=echantillons.id_echantillon" +
					" inner join stations on echantillons.id_station=stations.id_station" +
					" inner join lieux on stations.id_lieu=lieux.id_lieu" +
					" inner join liste_taxa_cpt on ligne_cpt.code_taxon=liste_taxa_cpt.code_taxon" +
					" left join synonymes on liste_taxa_cpt.code_taxon=synonymes.code_taxon" +
					" where nom_lieu like '" + lieu.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+"'"+ 
					" and nom_station like '"+ station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+"'"+
					" group by code_taxon"+
					" order by nom_taxon");

			/*		rs = query.executeQuery("select distinct code_taxon, nom_taxon, nom_auteur, code_taxon_ref, cf"+
					" from lieux natural join echantillons natural join stations natural join comptage natural join ligne_cpt natural join liste_taxa natural join auteurs natural join synonymes"+
					" where nom_lieu like '" + lieu.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+"'"+ 
					" and nom_station like '"+ station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+"'"+
					" group by code_taxon"+
					" order by nom_taxon");
			 */
			while( rs.next() )
			{
				data.add(rs.getString("code_taxon"));

				// gestion de la colonne Cf
				String col_cf = rs.getString("cf");
				if(col_cf.equals("1"))
					data.add("Cf.");
				else data.add("");

				data.add(rs.getString("nom_taxon"));
				data.add(rs.getString("nom_auteur"));
				data.add(rs.getString("code_taxon_ref"));

			}

			// prise en compte des taxons personnels !
	/*		rs = query.executeQuery("select code_taxon, nom_taxon, nom_auteur, cf"+
					" from lieux natural join echantillons natural join stations natural join comptage natural join ligne_cpt natural join liste_taxa_cpt natural join auteurs"+
					" where nom_lieu like '" + lieu.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+"'"+ 
					" and nom_station like '"+ station.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
					+"'"+
					" group by code_taxon"+
					" order by nom_taxon");

			while( rs.next() ) {
				data.add(rs.getString("code_taxon"));

				// gestion de la colonne Cf
				String col_cf = rs.getString("cf");
				if(col_cf.equals("1"))
					data.add("Cf.");
				else data.add("");

				data.add(rs.getString("nom_taxon"));
				data.add(rs.getString("nom_auteur"));
				data.add("");
			}
*/
			returned = new Object[data.size()/5][5];

			for( int i = 0; i < data.size(); i++)
				returned[i/5][i%5] = (Object)data.get(i);

		} 
		catch (SQLException e) {
			e.printStackTrace();
		}

		return returned;
	}

	/**
	 * Retourne la liste des sites, stations, chantillons contenant un taxon donn
	 * @return une matrice de donnes
	 */
	public Object[][] get_all_for_taxon(String code_taxon)
	{
		Object[][] returned = null;
		ArrayList<Object> data = new ArrayList<Object>();

		// test si le taxon est personnel : commence par 'NEW'
		boolean perso = code_taxon.startsWith("NEW");

		try 
		{

			if (perso) {
				rs = query.executeQuery("select c.id_cpt, nom_lieu, nom_station, date_prelevement, ref_etude,"
						+ " l.cf, l.nb_cell_moy_ind, l.nb_cpt, l.type, l.biovol_calcule"
						+ " from echantillons e, stations s, comptage c, ligne_cpt l, liste_taxa_cpt li, lieux lx"
						+ " where e.id_station=s.id_station and lx.id_lieu=s.id_lieu"
						+ " and e.id_echantillon=c.id_echantillon and c.id_cpt=l.id_cpt and l.code_taxon=li.code_taxon"
						+ " and l.code_taxon like '" + code_taxon.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
						+ "'" + " order by nom_lieu, nom_station, date_prelevement");

			}
			else {
				rs = query.executeQuery("select c.id_cpt, nom_lieu, nom_station, date_prelevement, ref_etude,"
						+ " l.cf, l.nb_cell_moy_ind, l.nb_cpt, l.type, l.biovol_calcule"
						+ " from echantillons e, stations s, comptage c, ligne_cpt l, liste_taxa li, lieux lx"
						+ " where e.id_station=s.id_station and lx.id_lieu=s.id_lieu"
						+ " and e.id_echantillon=c.id_echantillon and c.id_cpt=l.id_cpt and l.code_taxon=li.code_taxon"
						+ " and l.code_taxon like '" + code_taxon.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'")
						+ "'" + " order by nom_lieu, nom_station, date_prelevement");
			}

			while( rs.next() )
			{
				data.add(rs.getInt("c.id_cpt"));
				data.add(rs.getString("nom_lieu"));
				data.add(rs.getString("nom_station"));
				data.add(rs.getString("date_prelevement"));
				data.add(rs.getString("ref_etude"));

				if(rs.getInt("l.cf") == 1)
					data.add("Cf.");
				else 
					data.add("");

				if (rs.getInt("l.nb_cell_moy_ind")==0)
					data.add(null);
				else
					data.add(rs.getInt("l.nb_cell_moy_ind"));

				data.add(rs.getInt("l.nb_cpt"));
				data.add(rs.getString("l.type"));
				data.add(rs.getFloat("l.biovol_calcule"));

			}

			returned = new Object[data.size()/10][10];

			for( int i = 0; i < data.size(); i++)
				returned[i/10][i%10] = (Object)data.get(i);

		} 
		catch (SQLException e) {
			e.printStackTrace();
		}

		return returned;
	}

	/**
	 * Retourne le code du dernier taxon cr par un utilisateur
	 * @return code du taxon (du type NEWxxx -> 001 <= xxx <= 999)
	 */
	public String get_last_cpt_taxon()
	{
		String returned = null;

		try
		{
			rs = query.executeQuery("SELECT code_taxon FROM liste_taxa_cpt WHERE code_taxon LIKE 'NEW%' ORDER BY code_taxon DESC LIMIT 1");
			if( rs.next() )
			{
				returned = rs.getString(1);

				int suff = Integer.parseInt(returned.substring(3));
				suff++;
				if( suff < 10 )
					returned = "NEW00" + suff;
				else
				{
					if( suff < 100 )
						returned = "NEW0" + suff;
					else
						returned = "NEW" + suff;
				}
			}
			else
				returned = "NEW001";
		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}

	/**
	 * Ajoute un taxon utilisateur
	 * @param code_taxon code du taxon 
	 * @param nom_taxon nom du taxon 
	 * @param nom_auteur nom de l'auteur
	 * @param indiv individu par dfaut
	 * @param biovolcel biovolume cellulaire
	 * @param biovolind biovolume par individu
	 * @param embranchement du taxon
	 * @param createur nom du crateur
	 */
	public void add_taxon_cpt(String code_taxon, String nom_taxon, String nom_auteur, String indiv, float biovolcel,
			float biovolind, String embr, String createur, String formeCel, String formeInd)
	{
		Object idCel = null, idInd = null;

		if(formeCel.compareTo("") !=0 ){
			try
			{
				rs = query.executeQuery("SELECT id_forme AS id_forme_cel "
						+ "FROM formes "
						+ "WHERE nom_forme ='"+formeCel+"'");
				if( rs.next() )
				{	
					idCel = rs.getInt("id_forme_cel");	
				}
				else
				{
					idCel = null;
				}
			}
			catch(SQLException e) { e.printStackTrace(); }
		}

		if(formeInd.compareTo("") !=0 ){
			try
			{
				rs = query.executeQuery("SELECT id_forme AS id_forme_ind "
						+ "FROM formes "
						+ "WHERE nom_forme ='"+formeInd+"'");
				if( rs.next() )
				{	
					idInd = rs.getInt("id_forme_ind");	
				}
				else
				{
					idInd = null;
				}
			}
			catch(SQLException e) { e.printStackTrace(); }
		}

		try
		{
			query.executeUpdate("INSERT INTO liste_taxa_cpt VALUES ( NULL, '" + code_taxon + "', '" 
					+ nom_taxon.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "', '" +
					nom_auteur.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "', '" + indiv + "', " + 
					((biovolcel != 0) ? biovolcel : "NULL") + ", " + ((biovolind != 0) ? biovolind : "NULL") + ", " +
					(!embr.equals("") ? "'" + embr.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") +"'" : "NULL") + ", " +
					((createur.isEmpty()) ? "NULL" : "'" + createur.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "" +
							"\\\\'") + "'") + ","+idCel+", "+idInd+")");
		}
		catch(SQLException e) { e.printStackTrace(); }
	}

	/**
	 * Retourne les auteurs dont le nom commence par debut
	 * @param debut debut du nom d'auteur
	 * @return tableau d'auteurs
	 */
	public String[] get_auteurs_like( String debut )
	{
		String[] returned = null;
		ArrayList<String> tmp = new ArrayList<String>();

		try
		{
			rs = query.executeQuery("SELECT nom_auteur "
					+ "FROM auteurs "
					+ "WHERE nom_auteur LIKE '%" + debut + "%' " +
					"UNION "
					+ "SELECT nom_auteur "
					+ "FROM liste_taxa_cpt "
					+ "WHERE nom_auteur LIKE '%" + debut + "%' "
					+ "ORDER BY nom_auteur");
			while(rs.next())
				tmp.add(rs.getString(1));

			if( tmp.size() > 0)
			{
				returned = new String[tmp.size()];
				for( int i = 0; i < tmp.size(); i++ )
					returned[i] = tmp.get(i);
			}
		}
		catch(SQLException e) { e.printStackTrace(); }

		return returned;
	}


	/**
	 * 
	 * @return Un tableau de String correspondant  tous les embranchements distincts 
	 * des taxons personnels existants
	 */
	public String[] get_embranchements(){

		String[] liste_embranchements = null;
		ArrayList<String> tmp = new ArrayList<String>();

		try{
			rs = query.executeQuery("SELECT DISTINCT embranchement "
					+ "FROM taxinomie "
					+ "ORDER BY embranchement;");

			while(rs.next()){
				if (rs.getString(1) != null)
					tmp.add(rs.getString(1));
			}

			if (tmp.size()>0){

				liste_embranchements = new String[tmp.size()];

				for(int i =0; i<tmp.size(); ++i)
					liste_embranchements[i] = tmp.get(i);
			}

		}
		catch(SQLException e) { e.printStackTrace(); }

		return liste_embranchements;
	}

	/**
	 * 
	 * @param debut_embr
	 * @return Une liste de String correspondant aux embranchements distincts qui commencent
	 * par les lettres que l'utilisateur  saisies dans le champ embranchement.
	 * 
	 */
	public String[] get_embranchement_designes(String debut_embr){
		String[] liste_embr_designes = null;
		ArrayList<String> tmp = new ArrayList<String>();

		try{

			rs = query.executeQuery(
					"SELECT DISTINCT embranchement " +
							"FROM taxinomie " + 
							"WHERE embranchement LIKE '" + debut_embr.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + "%' " +
					"ORDER BY embranchement");

			while(rs.next()){
				if (rs.getString(1) != null)
					tmp.add(rs.getString(1));
			}

			if (tmp.size()>0){

				liste_embr_designes = new String[tmp.size()];

				for(int i =0; i<tmp.size(); ++i)
					liste_embr_designes[i] = tmp.get(i);
			}

		}
		catch (SQLException e) { e.printStackTrace(); }

		return liste_embr_designes;
	}


	/**
	 * 
	 * @param code du taxon valid
	 * @return l'embranchement d'un taxon valid donn
	 */
	public String get_embranchement(String code) {
		String emb = "";
		try{
			rs = query.executeQuery(
					"SELECT embranchement " +
							"FROM liste_taxa " +
							"NATURAL JOIN taxinomie " + 
							"WHERE code_taxon LIKE '" + code + "'");

			if (rs.next()){
				if (rs.getString(1) != null)
					emb = rs.getString(1);
			}
		}
		catch (SQLException e) { e.printStackTrace(); }

		return emb;
	}


	/**
	 * Ferme les connections
	 */
	public void close()
	{
		try 
		{
			query.close();
			connection.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public void change_isUser(boolean b, int idCpt, int idLine) {
		try
		{
			query.executeUpdate("UPDATE ligne_cpt SET taxon_cpt = " + ((b)? "1" : "0") + " WHERE id_cpt = " + idCpt + " AND id_ligne = " + idLine);
		}
		catch(SQLException e){ e.printStackTrace(); };
	}

	public void set_commentaire(int idCpt, String com) 
	{
		try 
		{
			query.executeUpdate("UPDATE comptage SET commentaire = '" + 
					com.replaceAll("\\\\", "\\\\\\\\").replaceAll("'", "\\\\'") + 
					"' WHERE id_cpt = " + idCpt);

		} catch (SQLException e) { e.printStackTrace(); }
	}

	// fonction autorisant l'ouverture d'un comptage si date cpt < date mise a jour liste taxa
	// NON UTILISEE
	public Date get_dateCpt(int idCpt) 
	{
		Date dateCpt=null;
		try 
		{
			rs = query.executeQuery("SELECT date_cpt FROM comptage WHERE id_cpt = " + idCpt);
			rs.next();
			dateCpt = rs.getDate(1);
		} catch (SQLException e) { e.printStackTrace(); }

		return dateCpt;
	}
}
