package Comptage;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.text.DecimalFormat;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

@SuppressWarnings("serial")
public class Consultation_Table extends JTable {

	@SuppressWarnings("unused")
	private Locale locale;
	private Printable p = null;
	private JPanel panel;
	
	public Consultation_Table(Locale locale)
	{
		this.locale = locale;
		ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		
		String[] columns = {rb.getString("colCodeSandre1"), rb.getString("colCodeTaxon1"), 
				rb.getString("colTypeObj1"), rb.getString("colCf"), 
				rb.getString("colNomTaxon"), rb.getString("colAuteur"), 
				rb.getString("colCodeRef1"), rb.getString("colNiveauTaxinomique"), 
				rb.getString("colNbCel_ml1"), rb.getString("colNbInd_ml1"),
				rb.getString("colBiovol_mm3_L1")};
		
		Object[][] rows = { };
		
		DefaultTableModel model = new DefaultTableModel(rows, columns) {
	        @SuppressWarnings({ "unchecked", "rawtypes" })
			public Class getColumnClass(int c) {
	            return getValueAt(0, c).getClass();
	        }
	        public boolean isCellEditable(int row, int col) { return false; }
		};
		
		TableCellRenderer doubleRenderer = new DefaultTableCellRenderer() {
			
			@Override
			public Component getTableCellRendererComponent(JTable table,
					Object value, boolean isSelected, boolean hasFocus,
					int row, int column) {
				
				if(value instanceof Integer)
				{					
					if( (Integer)value == Integer.MAX_VALUE )
						return super.getTableCellRendererComponent(table, "",
								isSelected, hasFocus, row, column);
				}
				
				return super.getTableCellRendererComponent(table, value,
						isSelected, hasFocus, row, column);
			}
		};
		
		this.setModel(model);
		this.getTableHeader().setReorderingAllowed(false);
		this.setAutoCreateRowSorter(true);
		// possibilit d'ajuster manuellement la largeur des colonnes
		this.getTableHeader().setResizingAllowed(true);
		this.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		this.setRowHeight(this.getRowHeight() + 5);
		this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		// appel du formattage de certaines colonnes
		this.getColumnModel().getColumn(10).setCellRenderer(new DecimalFormatRenderer());
		this.getColumnModel().getColumn(9).setCellRenderer(new DecimalFormatRenderer());
		this.getColumnModel().getColumn(8).setCellRenderer(new DecimalFormatRenderer());
		this.getColumnModel().getColumn(0).setCellRenderer(doubleRenderer);
		
		// ajustement de la largeur des colonnes
		this.getColumnModel().getColumn(0).setPreferredWidth(55);
		this.getColumnModel().getColumn(1).setPreferredWidth(60);
		this.getColumnModel().getColumn(2).setPreferredWidth(35);
		this.getColumnModel().getColumn(3).setPreferredWidth(30);
		this.getColumnModel().getColumn(4).setPreferredWidth(270);
		this.getColumnModel().getColumn(5).setPreferredWidth(223);
		this.getColumnModel().getColumn(6).setPreferredWidth(60);
		this.getColumnModel().getColumn(7).setPreferredWidth(160);
		this.getColumnModel().getColumn(8).setPreferredWidth(70);
		this.getColumnModel().getColumn(9).setPreferredWidth(70);
		this.getColumnModel().getColumn(10).setPreferredWidth(70);	
	}
	
	
	
	// Formatage pour le nombre de chiffres significatifs, et l'alignement  droite
	// colonnes concernes : les deux colonnes d'abondances, et la colonne du biovolume
	static class DecimalFormatRenderer extends DefaultTableCellRenderer {
		
		private static final long serialVersionUID = 1L;
		
		
		//dfinition des formats dcimals des colonnes de la JTable		
		// format du biovolume : 5 chiffres aprs la virgule
		DecimalFormat formatterBV = new DecimalFormat( "#0.00000");
		
		// format des abondances : 2 chiffres aprs la virgule
		private static final DecimalFormat formatterAb = new DecimalFormat( "#0.00" );
		
		
		public Component getTableCellRendererComponent(
		JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				
			// colonnne des abondances
			if(column == 8 || column == 9){
				
				// alignement  droite des valeurs dans la colonne
				this.setHorizontalAlignment(SwingConstants.RIGHT);
				
				// les abondances valant -1 ou 0 ne sont pas affiches
				if((Float)value == -1f || (Float)value == 0f ){
					return super.getTableCellRendererComponent(table, "",
							isSelected, hasFocus, row, column);
				}
				// les abondances valides sont mises au bon format dcimal
				else {
					value = formatterAb.format((Number)value);
					return super.getTableCellRendererComponent(
							table, value, isSelected, hasFocus, row, column );
				}
			}
				
			// colonne du biovolume calcul
			if(column == 10){
				// alignement  droite	
				this.setHorizontalAlignment(SwingConstants.RIGHT);
					
				// les cases de la colonne du biovolume = 0 sont masques
				if((Float)value == 0.0f){
					return super.getTableCellRendererComponent(table, "",
							isSelected, hasFocus, row, column);
				}
				// les biovolumes corrects sont mis au bon format
				else {
					value = formatterBV.format((Number)value);
					return super.getTableCellRendererComponent(
							table, value, isSelected, hasFocus, row, column );
				}
			}
			
			// valeur retourne par dfaut...
			return super.getTableCellRendererComponent(
				table, value, isSelected, hasFocus, row, column );
		} 
	}

	public void setJPanel(JPanel p)
	{
		this.panel = p;
	}
	
	public Printable getPrintable(PrintMode printMode, MessageFormat headerFormat, MessageFormat footerFormat)
	{
		MessageFormat pageNumber = new MessageFormat("- {0} -");
		this.p = super.getPrintable(printMode, null, pageNumber);
		return new FancyPrintable(this.p, this.panel);
	}
}


class FancyPrintable implements Printable {
	
	private Printable p;
	private JPanel panel;

	public FancyPrintable(Printable p, JPanel panel) {
		this.p = p;
		this.panel = panel;
	}

	public int print(Graphics graphics, final PageFormat pageFormat, int pageIndex)
			throws PrinterException {
		
		panel.setBorder(null);
		
		final BufferedImage scaledImage = new BufferedImage((int)(451 * 1.54), (int)(110 * 1.54), 
				BufferedImage.TYPE_INT_RGB);
		
		BufferedImage bi = new BufferedImage(panel.getWidth(), panel.getHeight(), 
				BufferedImage.TYPE_INT_RGB);
		
		panel.paintAll(bi.getGraphics());
		
		graphics.drawImage(bi, 0, 0, null);
		
		PageFormat pf = new PageFormat() {
			public double getImageableX() {return pageFormat.getImageableX();}
			public double getImageableY() {return pageFormat.getImageableY() + scaledImage.getHeight()/2;}
            public double getImageableWidth() {return pageFormat.getImageableWidth();}
            public double getImageableHeight() {return pageFormat.getImageableHeight();}         
		};
		
		if( p.print(graphics, pf, pageIndex) == NO_SUCH_PAGE )
			return NO_SUCH_PAGE;
		
		return PAGE_EXISTS;
	}
	
}
