package Comptage;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

import Import_Export.Import_Exporter;

public class Synthese extends JDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Locale locale;
	private String db_link;
	private Comptage_Query cpt_query;
	private JTable listeTax = null;
	private JLabel station_lbl = new JLabel();
	private JButton synthese_sta = new JButton();
	private JMenuBar menu_bar = new JMenuBar();
	private JMenu file_menu = new JMenu(); 
	private JMenuItem close_menu_item = new JMenuItem();
	private JMenuItem export = new JMenuItem();
	private JComboBox<String> site_filter = new JComboBox<String>();
	private JComboBox<String> st_filter = new JComboBox<String>();
	private boolean throwable = true;
	private Synthese self = null;

	public Synthese(Locale locale, String db_link)
	{
		this.locale = locale;
		this.db_link = db_link;
		this.setTitle("");

		cpt_query = new Comptage_Query(db_link);
		
		this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		this.getContentPane().setLayout(new BoxLayout(this.getContentPane(), BoxLayout.LINE_AXIS));
		self = this;
		this.translate(locale);
		add_buttons();
		add_handlers();
		add_menu();
		this.pack();
		this.setLocation(100, 100);
		self.setMinimumSize(new Dimension(500, 200));
		this.setResizable(false);
		this.setVisible(true);
		site_filter.setEditable(true);
		st_filter.setEditable(true);

	}

	private void add_menu()
	{
		file_menu.add(export);
		file_menu.add(close_menu_item);
		menu_bar.add(file_menu);
		this.setJMenuBar(menu_bar);
	}

	private void add_handlers()
	{
		
		// traduction des noms des colonnes
		final ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		final String[] columnNames = {rb.getString("colCodeTaxon"), rb.getString("colCf"), 
									rb.getString("colNomTaxon"), rb.getString("colAuteur"), 
									rb.getString("colCodeRef")};
		
		
		
		close_menu_item.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						self.dispose();
					}});	

		export.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						String site = (String) site_filter.getSelectedItem();
						String station = (String) st_filter.getSelectedItem();
						Import_Exporter synthSta = new Import_Exporter(locale, db_link);
						synthSta.export_synth_station(site, station);
					}});	

		//choix du site pour lequel on veut la synthse des taxons d'une station
		site_filter.getEditor().getEditorComponent().addKeyListener(new KeyListener() {
			String user_input;
			String[] sites;

			public void keyPressed(KeyEvent arg0) {
			}

			public void keyReleased(KeyEvent e) {
				if (e.getKeyCode() != KeyEvent.VK_DOWN
						&& e.getKeyCode() != KeyEvent.VK_BACK_SPACE
						&& e.getKeyCode() != KeyEvent.VK_UP) {
					user_input = ((JTextField) e.getSource()).getText().trim();
					sites = cpt_query.get_sites(user_input.replaceAll("'",
							"\\\\'"));
					throwable = false;
					if (sites != null) {
						self.site_filter.removeAllItems();
						self.st_filter.removeAllItems();
						if (sites.length > 1) {
							self.site_filter.addItem(user_input);
							for (String s : sites)
								self.site_filter.addItem(s);
							self.site_filter.hidePopup();
							self.site_filter.showPopup();
						} else {
							throwable = true;
							self.site_filter.addItem(sites[0]);
						}

					} else {
						throwable = false;
						self.site_filter.removeAllItems();
						self.st_filter.removeAllItems();
						self.site_filter.addItem(user_input);
						self.site_filter.addItem("");
					}
				} else {
					throwable = false;
					self.st_filter.removeAllItems();
				}
				throwable = true;
			}

			public void keyTyped(KeyEvent e) {
			}
		});

		//affichage des stations du site pralablement choisi
		site_filter.addItemListener(new ItemListener() {
			String selected;
			String[] stations;

			@SuppressWarnings("unchecked")
			public void itemStateChanged(ItemEvent e) {
				if (throwable) {
					throwable = false;
					self.st_filter.removeAllItems();
					selected = (String) ((JComboBox<String>) e.getSource())
							.getSelectedItem();
					if (selected != null) {
						stations = cpt_query.get_stations(selected);
						if (stations != null) {
							for (String s : stations)
								self.st_filter.addItem(s);

						} else
							self.st_filter.addItem("");
					} else {
						st_filter.removeAllItems();
					}
				}
				throwable = true;
			}
		});

		//affichage des taxons de la station et du site pralablement choisi par le  bouton
		synthese_sta.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						String site = null;
						String station = null;
						site = (String) site_filter.getSelectedItem();
						station = (String) st_filter.getSelectedItem();
						System.out.println(station);
						if (site != null || station != null )
						{
							// ouverture de la synthese sur une station
							((DefaultTableModel) listeTax.getModel()).setDataVector(cpt_query.get_taxon(site, station), columnNames);
							setTableFormat();
						}
						else
						{
							new JOptionPane();
							//message car selection de 0 site et/ou de 0 station
							JOptionPane.showMessageDialog(self, rb.getString("mssgSiteStationVide"),
									rb.getString("titreBoxSiteStationVide"), JOptionPane.INFORMATION_MESSAGE);

						}

					}
				});


		//affichage des taxons de la station et du site pralablement choisi par touche entre sur champ station
		st_filter.getEditor().getEditorComponent().addKeyListener(
				new KeyListener() {
					public void keyPressed(KeyEvent e) {
						String site = null;
						String station = null;
						site = (String) site_filter.getSelectedItem();
						station = (String) st_filter.getSelectedItem();
						
						if (e.getKeyCode() == KeyEvent.VK_ENTER) {
							((DefaultTableModel) listeTax.getModel()).setDataVector(cpt_query.get_taxon(site, station), columnNames);
							setTableFormat();
						}
					}

					public void keyReleased(KeyEvent arg0) {
					}

					public void keyTyped(KeyEvent arg0) {
					}
				});

	}

	private void add_buttons()
	{
		JPanel all = new JPanel();
		JPanel label = new JPanel();
		label.setLayout(new BoxLayout(label, BoxLayout.LINE_AXIS));
		label.setBackground(new Color(192, 197, 199));
		label.setAlignmentX(Component.LEFT_ALIGNMENT);
		all.setLayout(new BoxLayout(all, BoxLayout.PAGE_AXIS));
		station_lbl.setAlignmentX(Component.LEFT_ALIGNMENT);

		label.add(station_lbl);

		all.add(Box.createVerticalStrut(5));
		all.add(label);
		site_filter.setMaximumSize(new Dimension(250,20));
		site_filter.setPreferredSize(new Dimension(250,20));
		st_filter.setMaximumSize(new Dimension(250,20));
		st_filter.setPreferredSize(new Dimension(250,20));

		JPanel buttons_pane = new JPanel();
		buttons_pane.setLayout(new BoxLayout(buttons_pane, BoxLayout.LINE_AXIS));
		buttons_pane.setAlignmentX(Component.LEFT_ALIGNMENT);
		synthese_sta.setPreferredSize(new Dimension(100,20));
		buttons_pane.setBackground(new Color(192, 197, 199));
		buttons_pane.add(site_filter);
		buttons_pane.add(Box.createHorizontalStrut(5));
		buttons_pane.add(st_filter);
		buttons_pane.add(Box.createHorizontalGlue());
		buttons_pane.add(synthese_sta);
		buttons_pane.add(Box.createHorizontalStrut(40));

		all.add(Box.createVerticalStrut(5));
		all.add(buttons_pane);


		DefaultTableModel model = new DefaultTableModel(){
			/**
			 * cration de la table contenant la liste des taxons d'un site et d'une station
			 */
			private static final long serialVersionUID = 1L;

			public boolean isCellEditable(int rowIndex, int columnIndex) {
				return false;
			}
		};


		listeTax = new JTable(model);
		listeTax.getTableHeader().setReorderingAllowed(false);
		listeTax.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		listeTax.getTableHeader().setResizingAllowed(true);

		JScrollPane liste_scroll = new JScrollPane(listeTax, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		liste_scroll.setAlignmentX(Component.LEFT_ALIGNMENT);
		liste_scroll.setPreferredSize(new Dimension(700,600));
		all.add(Box.createVerticalStrut(5));
		all.add(liste_scroll);
		all.add(Box.createVerticalStrut(5));
		all.setBackground(new Color(192, 197, 199));	

		this.getContentPane().add(Box.createHorizontalStrut(5));
		this.getContentPane().add(all);
		this.getContentPane().add(Box.createHorizontalStrut(5));
		this.getContentPane().setBackground(new Color(192, 197, 199));

		listeTax.setAutoCreateRowSorter(true);

	}


	/**
	 * mthode pour dfinir la taille prfre de la Jtable liste des taxons
	 */
	private void setTableFormat(){
		listeTax.getColumnModel().getColumn(0).setPreferredWidth(80);
		listeTax.getColumnModel().getColumn(1).setPreferredWidth(25);
		listeTax.getColumnModel().getColumn(2).setPreferredWidth(260);
		listeTax.getColumnModel().getColumn(3).setPreferredWidth(235);
		listeTax.getColumnModel().getColumn(4).setPreferredWidth(80);

	}
	
	
	public void translate(Locale locale){
		
		this.locale = locale;
		ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		
		// nom de la fentre
		this.setTitle(rb.getString("titreFenSynthStation"));
		
		file_menu.setText(rb.getString("fichier"));

		export.setText(rb.getString("exporter"));
		close_menu_item.setText(rb.getString("fermer"));
		
		station_lbl.setText(rb.getString("saisieSitePuisStation"));
		
		synthese_sta.setText(rb.getString("synthese"));
		
		//ragencement de la position des items de la fentre
		this.pack();
	}

}
