package Comptage;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.text.DecimalFormat;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import Import_Export.Import_Exporter;

public class SyntheseTax extends JDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Locale locale;
	private String db_link;
	private Comptage_Query cpt_query;
	private JTable listeSta = null;
	private JButton synthese_tax = new JButton();
	private JMenuBar menu_bar = new JMenuBar();
	private JMenu file_menu = new JMenu(); 
	private JMenuItem close_menu_item = new JMenuItem();
	private JMenuItem export = new JMenuItem();
	
	private JLabel taxon_lbl = new JLabel();
	
	private JComboBox<String> code_filter = new JComboBox<String>();
	private JComboBox<String> nom_filter = new JComboBox<String>();
	private boolean throwable = true;
	private SyntheseTax self = null;

	public SyntheseTax(Locale locale, String db_link)
	{
		this.locale = locale;
		this.db_link = db_link;
		this.setTitle("Synthse pour un taxon");

		cpt_query = new Comptage_Query(db_link);
		
		this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		this.getContentPane().setLayout(new BoxLayout(this.getContentPane(), BoxLayout.LINE_AXIS));
		self = this;
		this.translate(locale);
		add_buttons();
		add_handlers();
		add_menu();
		this.pack();
		this.setLocation(100, 100);
		self.setMinimumSize(new Dimension(500, 200));
		this.setResizable(false);
		this.setVisible(true);
		code_filter.setEditable(true);
		nom_filter.setEditable(true);

	}

	private void add_menu()
	{
		file_menu.add(export);
		file_menu.add(close_menu_item);
		menu_bar.add(file_menu);
		this.setJMenuBar(menu_bar);
	}

	private void add_handlers()
	{
		
		// traduction des colonnes
		final ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		final String[] columnNames = {rb.getString("colNumCpt"), rb.getString("colNomSite"), 
				rb.getString("colNomStation"), rb.getString("colDatePrel"), 
					rb.getString("colRefEch"), rb.getString("colCf"), 
					rb.getString("colNbMoyCell_ind"), rb.getString("colNbCpt"),
					rb.getString("colType"), rb.getString("colBiovol_mm3_L")};
		
		
		close_menu_item.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						self.dispose();
					}});	

		export.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						String code = (String) code_filter.getSelectedItem();
						Import_Exporter synthTax = new Import_Exporter(locale, db_link);
						synthTax.export_synth_taxon(code);
					}});	

		//choix du code du taxon pour lequel on veut la synthse et affichage du nom correspondant
		((JTextField) code_filter.getEditor().getEditorComponent())
		.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (throwable) {
					throwable = false;
					((JTextField) e.getSource())
					.setText(((JTextField) e.getSource())
							.getText().trim().toUpperCase());
					nom_filter.removeAllItems();
					nom_filter.addItem(cpt_query
							.get_nom_taxon(((JTextField) e.getSource())
									.getText().trim()));
					nom_filter.setSelectedIndex(0);
					throwable = true;
				}
			}
		});

		code_filter.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent arg0) {
				if (arg0.getStateChange() == ItemEvent.SELECTED && throwable) {
					throwable = false;
					nom_filter.removeAllItems();
					nom_filter.addItem(cpt_query.get_nom_taxon((String) arg0
							.getItem()));
					nom_filter.setSelectedIndex(0);
					throwable = true;
				}
			}

		});

		code_filter.getEditor().getEditorComponent().addKeyListener(
				new KeyListener() {
					public void keyPressed(KeyEvent arg0) {
					}

					public void keyReleased(KeyEvent arg0) {
						String debut = ((JTextField) arg0.getSource())
								.getText().trim();
						if (arg0.getKeyCode() != KeyEvent.VK_BACK_SPACE
								&& arg0.getKeyCode() != KeyEvent.VK_ENTER
								&& arg0.getKeyCode() != KeyEvent.VK_DOWN
								&& arg0.getKeyCode() != KeyEvent.VK_UP
								&& debut.length() > 1) {
							throwable = false;
							((JTextField) nom_filter.getEditor().getEditorComponent())
							.setText("");
							nom_filter.removeAllItems();
							String[] codes = cpt_query.get_codes_taxon(debut);
							self.code_filter.removeAllItems();
							self.code_filter.addItem(debut);
							if (codes != null) {
								for (String s : codes)
									self.code_filter.addItem(s);
								self.code_filter.hidePopup();
								self.code_filter.showPopup();
							}
							throwable = true;
						}
					}

					public void keyTyped(KeyEvent arg0) {
					}
				});

		//choix du nom du taxon pour lequel on veut la synthse et affichage du code correspondant
		((JTextField) nom_filter.getEditor().getEditorComponent())
		.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (throwable) {
					throwable = false;
					code_filter.removeAllItems();
					code_filter.addItem(cpt_query
							.get_code_taxon_nom(((JTextField) e
									.getSource()).getText()));
					code_filter.setSelectedIndex(0);
					throwable = true;
				}
			}
		});

		nom_filter.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent arg0) {
				if (arg0.getStateChange() == ItemEvent.SELECTED && throwable) {
					throwable = false;
					code_filter.removeAllItems();
					code_filter.addItem(cpt_query.get_code_taxon_nom((String) arg0
							.getItem()));
					code_filter.setSelectedIndex(0);
					throwable = true;
				}
			}

		});
		
		
		// gestion de l'auto-slection du nom lorsque l'on commence  taper le code du taxon
		nom_filter.getEditor().getEditorComponent().addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent arg0) {
			}

			public void keyReleased(KeyEvent arg0) {
				String debut = ((JTextField) arg0.getSource()).getText();
				if (arg0.getKeyCode() != KeyEvent.VK_BACK_SPACE
						&& arg0.getKeyCode() != KeyEvent.VK_ENTER
						&& arg0.getKeyCode() != KeyEvent.VK_DOWN 
						&& arg0.getKeyCode() != KeyEvent.VK_UP
						&& debut.length() > 2) {
					
					// inactivation de la possibilit de modifs des champs noms et codes par java
					throwable = false;
					((JTextField) code_filter.getEditor().getEditorComponent())
					.setText("");
					code_filter.removeAllItems();
					String[] noms = cpt_query.get_noms_taxon_like(debut);
					self.nom_filter.removeAllItems();
					self.nom_filter.addItem(debut);
					if (noms != null) {
						// boucle for pour tous les items s du champs noms
						for (String s : noms)
							self.nom_filter.addItem(s);
						self.nom_filter.hidePopup();
						self.nom_filter.showPopup();
					}
					throwable = true;
				}
			}

			public void keyTyped(KeyEvent arg0) {
			}
		});


		//affichage des sites, des stations et des chantillons contenant le taxon pralablement choisi par le  bouton
		synthese_tax.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						String code_taxon = null;
						code_taxon = (String) code_filter.getSelectedItem();
						System.out.println(code_taxon);
						if (code_taxon != null )
						{
							// ouverture de la synthese pour un taxon
							((DefaultTableModel) listeSta.getModel()).setDataVector(cpt_query.get_all_for_taxon(code_taxon), columnNames);
							setTableFormat();
						}
						else
						{
							new JOptionPane();
							//message car selection de 0 taxon
							JOptionPane.showMessageDialog(self, rb.getString("mssgTaxonNonSaisit"),
									 rb.getString("titreBoxTaxonNonSaisit"), JOptionPane.INFORMATION_MESSAGE);

						}

					}
				});


		//affichage des sites, des stations et des chantillons contenant le taxon pralablement choisi par touche entre sur champ code
		code_filter.getEditor().getEditorComponent().addKeyListener(
				new KeyListener() {
					public void keyPressed(KeyEvent e) {
						String code_taxon = null;
						code_taxon = (String) code_filter.getSelectedItem();
						System.out.println(code_taxon);
						if (e.getKeyCode() == KeyEvent.VK_ENTER) {
							((DefaultTableModel) listeSta.getModel()).setDataVector(cpt_query.get_all_for_taxon(code_taxon), columnNames);
							setTableFormat();
						}
					}

					public void keyReleased(KeyEvent arg0) {
					}

					public void keyTyped(KeyEvent arg0) {
					}
				});

	}

	private void add_buttons()
	{

		JPanel all = new JPanel();
		JPanel label = new JPanel();
		label.setLayout(new BoxLayout(label, BoxLayout.LINE_AXIS));
		label.setBackground(new Color(192, 197, 199));
		label.setAlignmentX(Component.LEFT_ALIGNMENT);
		all.setLayout(new BoxLayout(all, BoxLayout.PAGE_AXIS));
		taxon_lbl.setAlignmentX(Component.LEFT_ALIGNMENT);

		label.add(taxon_lbl);

		all.add(Box.createVerticalStrut(5));
		all.add(label);
		code_filter.setMaximumSize(new Dimension(100,20));
		code_filter.setPreferredSize(new Dimension(100,20));
		nom_filter.setMaximumSize(new Dimension(250,20));
		nom_filter.setPreferredSize(new Dimension(250,20));

		JPanel buttons_pane = new JPanel();
		buttons_pane.setLayout(new BoxLayout(buttons_pane, BoxLayout.LINE_AXIS));
		buttons_pane.setAlignmentX(Component.LEFT_ALIGNMENT);
		synthese_tax.setPreferredSize(new Dimension(100,20));
		buttons_pane.setBackground(new Color(192, 197, 199));
		buttons_pane.add(code_filter);
		buttons_pane.add(Box.createHorizontalStrut(5));
		buttons_pane.add(nom_filter);
		buttons_pane.add(Box.createHorizontalGlue());
		buttons_pane.add(synthese_tax);
		buttons_pane.add(Box.createHorizontalStrut(40));

		all.add(Box.createVerticalStrut(5));
		all.add(buttons_pane);


		DefaultTableModel model = new DefaultTableModel(){
			/**
			 * cration de la table contenant la liste des sites et des stations contenant le taxon
			 */
			private static final long serialVersionUID = 1L;

			public boolean isCellEditable(int rowIndex, int columnIndex) {
				return false;
			}
		};
		listeSta = new JTable(model);
		listeSta.setModel(new DefaultTableModel(){
			/**
			 * dfinit le type des colonnes du tableau pour permettre le tri sur les nombres 
			 */
			private static final long serialVersionUID = 1L;
			@SuppressWarnings("rawtypes")
			Class[] types = new Class [] {
				java.lang.Integer.class, java.lang.String.class, java.lang.String.class, java.lang.String.class,
				java.lang.String.class, java.lang.Integer.class, java.lang.Integer.class, java.lang.Integer.class, java.lang.String.class,
				java.lang.Float.class
			};
			@SuppressWarnings({ "unchecked", "rawtypes" })
			public Class getColumnClass(int columnIndex) {
				return types [columnIndex];
			}
		});
		
		listeSta.getTableHeader().setReorderingAllowed(false);
		listeSta.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		listeSta.getTableHeader().setResizingAllowed(true);

		JScrollPane liste_scroll = new JScrollPane(listeSta, 
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		
		liste_scroll.setAlignmentX(Component.LEFT_ALIGNMENT);
		liste_scroll.setPreferredSize(new Dimension(825,600));
		all.add(Box.createVerticalStrut(5));
		all.add(liste_scroll);
		all.add(Box.createVerticalStrut(5));
		all.setBackground(new Color(192, 197, 199));	

		this.getContentPane().add(Box.createHorizontalStrut(5));
		this.getContentPane().add(all);
		this.getContentPane().add(Box.createHorizontalStrut(5));
		this.getContentPane().setBackground(new Color(192, 197, 199));

		listeSta.setAutoCreateRowSorter(true);

	}

	static class DecimalFormatRenderer extends DefaultTableCellRenderer {
		/**
		 * dfinit le format dcimal de colonne de la JTable
		 */
		private static final long serialVersionUID = 1L;
		private static final DecimalFormat formatter = new DecimalFormat( "#0.00000" );
		 
		public Component getTableCellRendererComponent(
		JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
		// d'abord dfinir le format de la valeur de la cell comme requis 
		value = formatter.format((Number)value);
		
		// colonne du biovolume calcul
		if(column == 9){
			// alignement  droite	
			this.setHorizontalAlignment(SwingConstants.RIGHT);
		}
		
		
		// et passer cela  la classe parent
		return super.getTableCellRendererComponent(
		table, value, isSelected, hasFocus, row, column );
		} 
	}
	
	/**
	 * mthode pour dfinir la taille prfre de la Jtable liste des sites stations
	 * dfinition du format decimal de la colonne biovol
	 */
	private void setTableFormat(){
		listeSta.getColumnModel().getColumn(0).setPreferredWidth(45);
		listeSta.getColumnModel().getColumn(1).setPreferredWidth(160);
		listeSta.getColumnModel().getColumn(2).setPreferredWidth(160);
		listeSta.getColumnModel().getColumn(3).setPreferredWidth(70);
		listeSta.getColumnModel().getColumn(4).setPreferredWidth(80);
		listeSta.getColumnModel().getColumn(5).setPreferredWidth(30);
		listeSta.getColumnModel().getColumn(6).setPreferredWidth(80);
		listeSta.getColumnModel().getColumn(7).setPreferredWidth(80);
		listeSta.getColumnModel().getColumn(8).setPreferredWidth(32);
		listeSta.getColumnModel().getColumn(9).setPreferredWidth(80);

		listeSta.getColumnModel().getColumn(9).setCellRenderer(new DecimalFormatRenderer() );
		
		// ajustement de la hauteur de l'entte de la table
		int hauteur_entete = 35;
		listeSta.getTableHeader().setPreferredSize(new Dimension(listeSta.getTableHeader().getWidth(), hauteur_entete));

	}
	
	
	/**
	 * Traduction dans la bonne langue de tous les textes de la fentre
	 * @param locale
	 */
	public void translate(Locale locale){
		this.locale = locale;
		ResourceBundle rb = ResourceBundle.getBundle("ressources.Langue", locale);
		
		// nom de la fentre
		this.setTitle(rb.getString("titreFenSynthTaxon"));
		
		file_menu.setText(rb.getString("fichier"));
		export.setText(rb.getString("exporter"));
		close_menu_item.setText(rb.getString("fermer"));
		
		taxon_lbl.setText(rb.getString("saisieCodeNomTaxon"));
		
		synthese_tax.setText(rb.getString("synthese"));
		
		//ragencement de la position des items de la fentre
		this.pack();
	}
	

}
